# Generated by Django 1.10.2 on 2016-12-12 00:58

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MumbleUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(max_length=254, unique=True)),
                ('pwhash', models.CharField(max_length=40)),
                ('groups', models.TextField(blank=True, null=True)),
            ],
            options={
                'db_table': 'services_mumbleuser',
            },
        ),
    ]



from django.urls import include, path

from . import views

app_name = 'xenforo'

module_urls = [
    # XenForo service control
    path('activate/', views.activate_xenforo_forum, name='activate'),
    path('deactivate/', views.deactivate_xenforo_forum, name='deactivate'),
    path('reset_password/', views.reset_xenforo_password, name='reset_password'),
    path('set_password/', views.set_xenforo_password, name='set_password'),
]

urlpatterns = [
    path('xenforo/', include((module_urls, app_name), namespace=app_name)),
]



# Generated by Django 3.2.8 on 2021-10-20 13:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("timerboard", "0003_on_delete"),
    ]

    operations = [
        migrations.AddField(
            model_name="timer",
            name="timer_type",
            field=models.CharField(
                choices=[
                    ("UNSPECIFIED", "Not Specified"),
                    ("SHIELD", "Shield"),
                    ("ARMOR", "Armor"),
                    ("HULL", "Hull"),
                    ("FINAL", "Final"),
                    ("ANCHORING", "Anchoring"),
                    ("UNANCHORING", "Unanchoring"),
                ],
                default="UNSPECIFIED",
                max_length=254,
            ),
        ),
    ]



"""Counters."""

from typing import Optional

from redis import Redis

from django.core.cache import cache

from .cache import get_redis_client


class ItemCounter:
    """A process safe item counter.

    Args:
        - name: Unique name for the counter
        - minimum: Counter can not go below the minimum, when set
        - redis: A Redis client. Will use AA's cache client by default
    """

    CACHE_KEY_BASE = "allianceauth-item-counter"
    DEFAULT_CACHE_TIMEOUT = 24 * 3600

    def __init__(
        self, name: str, minimum: Optional[int] = None, redis: Optional[Redis] = None
    ) -> None:
        if not name:
            raise ValueError("Must define a name")

        self._name = str(name)
        self._minimum = minimum
        self._redis = get_redis_client() if not redis else redis

    @property
    def _cache_key(self) -> str:
        return f"{self.CACHE_KEY_BASE}-{self._name}"

    def reset(self, init_value: int = 0):
        """Reset counter to initial value."""
        with self._redis.lock(f"{self.CACHE_KEY_BASE}-reset"):
            if self._minimum is not None and init_value < self._minimum:
                raise ValueError("Can not reset below minimum")

            cache.set(self._cache_key, init_value, self.DEFAULT_CACHE_TIMEOUT)

    def incr(self, delta: int = 1):
        """Increment counter by delta."""
        try:
            cache.incr(self._cache_key, delta)
        except ValueError:
            pass

    def decr(self, delta: int = 1):
        """Decrement counter by delta."""
        with self._redis.lock(f"{self.CACHE_KEY_BASE}-decr"):
            if self._minimum is not None and self.value() == self._minimum:
                return
            try:
                cache.decr(self._cache_key, delta)
            except ValueError:
                pass

    def value(self) -> Optional[int]:
        """Return current value or None if not yet initialized."""
        return cache.get(self._cache_key)



# Generated by Django 1.11.5 on 2017-10-07 03:55

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    replaces = [('services', '0001_initial'), ('services', '0002_auto_20161016_0135'), ('services', '0003_delete_groupcache')]

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
    ]



import logging

from django.apps import AppConfig
from django.db.utils import ProgrammingError, OperationalError

logger = logging.getLogger(__name__)


class MenuConfig(AppConfig):
    name = "allianceauth.menu"
    label = "menu"

    def ready(self):
        try:
            logger.debug("Syncing MenuItem Hooks")
            from allianceauth.menu.providers import MenuItem
            MenuItem.sync_hook_models()
        except (ProgrammingError, OperationalError):
            logger.warning("Migrations not completed for MenuItems")



from allianceauth.analytics.models import AnalyticsIdentifier
from django.core.exceptions import ValidationError

from django.test.testcases import TestCase

from uuid import UUID, uuid4


# Identifiers
uuid_1 = "ab33e241fbf042b6aa77c7655a768af7"
uuid_2 = "7aa6bd70701f44729af5e3095ff4b55c"


class TestAnalyticsIdentifier(TestCase):

    def test_identifier_random(self):
        self.assertNotEqual(AnalyticsIdentifier.objects.get(), uuid4)

    def test_identifier_singular(self):
        AnalyticsIdentifier.objects.all().delete()
        AnalyticsIdentifier.objects.create(identifier=uuid_1)
        # Yeah i have multiple asserts here, they all do the same thing
        with self.assertRaises(ValidationError):
            AnalyticsIdentifier.objects.create(identifier=uuid_2)
        self.assertEqual(AnalyticsIdentifier.objects.count(), 1)
        self.assertEqual(AnalyticsIdentifier.objects.get(
            pk=1).identifier, UUID(uuid_1))



from unittest.mock import patch

from celery.exceptions import Retry

from django.test import TestCase, override_settings

from allianceauth.authentication.task_statistics.counters import (
    failed_tasks,
    retried_tasks,
    succeeded_tasks,
)
from allianceauth.authentication.task_statistics.signals import (
    reset_counters,
    is_enabled,
)
from allianceauth.eveonline.tasks import update_character


@override_settings(
    CELERY_ALWAYS_EAGER=True, ALLIANCEAUTH_DASHBOARD_TASK_STATISTICS_DISABLED=False
)
class TestTaskSignals(TestCase):
    fixtures = ["disable_analytics"]

    def setUp(self) -> None:
        succeeded_tasks.clear()
        retried_tasks.clear()
        failed_tasks.clear()

    def test_should_record_successful_task(self):
        # when
        with patch(
            "allianceauth.eveonline.tasks.EveCharacter.objects.update_character"
        ) as mock_update:
            mock_update.return_value = None
            update_character.delay(1)
        # then
        self.assertEqual(succeeded_tasks.count(), 1)
        self.assertEqual(retried_tasks.count(), 0)
        self.assertEqual(failed_tasks.count(), 0)

    def test_should_record_retried_task(self):
        # when
        with patch(
            "allianceauth.eveonline.tasks.EveCharacter.objects.update_character"
        ) as mock_update:
            mock_update.side_effect = Retry
            update_character.delay(1)
        # then
        self.assertEqual(succeeded_tasks.count(), 0)
        self.assertEqual(failed_tasks.count(), 0)
        self.assertEqual(retried_tasks.count(), 1)

    def test_should_record_failed_task(self):
        # when
        with patch(
            "allianceauth.eveonline.tasks.EveCharacter.objects.update_character"
        ) as mock_update:
            mock_update.side_effect = RuntimeError
            update_character.delay(1)
        # then
        self.assertEqual(succeeded_tasks.count(), 0)
        self.assertEqual(retried_tasks.count(), 0)
        self.assertEqual(failed_tasks.count(), 1)

    def test_should_reset_counters(self):
        # given
        succeeded_tasks.add()
        retried_tasks.add()
        failed_tasks.add()
        # when
        reset_counters()
        # then
        self.assertEqual(succeeded_tasks.count(), 0)
        self.assertEqual(retried_tasks.count(), 0)
        self.assertEqual(failed_tasks.count(), 0)


class TestIsEnabled(TestCase):
    @override_settings(ALLIANCEAUTH_DASHBOARD_TASK_STATISTICS_DISABLED=False)
    def test_enabled(self):
        self.assertTrue(is_enabled())

    @override_settings(ALLIANCEAUTH_DASHBOARD_TASK_STATISTICS_DISABLED=True)
    def test_disabled(self):
        self.assertFalse(is_enabled())



import random
import string
import calendar
from datetime import datetime
import hashlib
import logging
import re
from typing import Tuple

from packaging import version

from django.db import connections
from django.conf import settings
from django.contrib.auth.models import User

from allianceauth.eveonline.models import EveCharacter

logger = logging.getLogger(__name__)


TABLE_PREFIX = getattr(settings, 'SMF_TABLE_PREFIX', 'smf_')


class SmfManager:
    def __init__(self):
        pass

    # For SMF < 2.1
    SQL_ADD_USER_SMF_20 = r"INSERT INTO %smembers (member_name, passwd, email_address, date_registered, real_name," \
                            r" buddy_list, message_labels, openid_uri, signature, ignore_boards) " \
                            r"VALUES (%%s, %%s, %%s, %%s, %%s, 0, 0, 0, 0, 0)" % TABLE_PREFIX

    # For SMF >= 2.1
    SQL_ADD_USER_SMF_21 = r"INSERT INTO %smembers (member_name, passwd, email_address, date_registered, real_name," \
                            r" buddy_list, signature, ignore_boards) " \
                            r"VALUES (%%s, %%s, %%s, %%s, %%s, 0, 0, 0)" % TABLE_PREFIX

    # returns something like »window.smfVersion = "SMF 2.0.19";«
    SQL_GET_CURRENT_SMF_VERSION = r"SELECT data FROM %sadmin_info_files WHERE filename = %%s" % TABLE_PREFIX

    SQL_DEL_USER = r"DELETE FROM %smembers where member_name = %%s" % TABLE_PREFIX

    SQL_UPD_USER = r"UPDATE %smembers SET email_address = %%s, passwd = %%s, real_name = %%s WHERE member_name = %%s" % TABLE_PREFIX

    SQL_UPD_DISPLAY_NAME = r"UPDATE %smembers SET real_name = %%s WHERE member_name = %%s" % TABLE_PREFIX

    SQL_DIS_USER = r"UPDATE %smembers SET email_address = %%s, passwd = %%s WHERE member_name = %%s" % TABLE_PREFIX

    SQL_USER_ID_FROM_USERNAME = r"SELECT id_member from %smembers WHERE member_name = %%s" % TABLE_PREFIX

    SQL_ADD_USER_GROUP = r"UPDATE %smembers SET additional_groups = %%s WHERE id_member = %%s" % TABLE_PREFIX

    SQL_GET_GROUP_ID = r"SELECT id_group from %smembergroups WHERE group_name = %%s" % TABLE_PREFIX

    SQL_ADD_GROUP = r"INSERT INTO %smembergroups (group_name,description) VALUES (%%s,%%s)" % TABLE_PREFIX

    SQL_UPDATE_USER_PASSWORD = r"UPDATE %smembers SET passwd = %%s WHERE member_name = %%s" % TABLE_PREFIX

    SQL_REMOVE_USER_GROUP = r"UPDATE %smembers SET additional_groups = %%s WHERE id_member = %%s" % TABLE_PREFIX

    SQL_GET_ALL_GROUPS = r"SELECT id_group, group_name FROM %smembergroups" % TABLE_PREFIX

    SQL_GET_USER_GROUPS = r"SELECT additional_groups FROM %smembers WHERE id_member = %%s" % TABLE_PREFIX

    SQL_ADD_USER_AVATAR = r"UPDATE %smembers SET avatar = %%s WHERE id_member = %%s" % TABLE_PREFIX

    @classmethod
    def _get_current_smf_version(cls) -> str:
        """
        Get the current SMF version from the DB
        :return:
        """

        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_GET_CURRENT_SMF_VERSION, ['current-version.js'])
        row = cursor.fetchone()
        db_result = row[0]

        pattern = re.compile(r"\d+(\.\d+)+")
        result = pattern.search(db_result)

        smf_version = result.group(0)

        return smf_version


    @staticmethod
    def _sanitize_groupname(name):
        name = name.strip(' _')
        return re.sub(r'[^\w.-]', '', name)

    @staticmethod
    def generate_random_pass():
        return ''.join([random.choice(string.ascii_letters + string.digits) for n in range(16)])

    @staticmethod
    def gen_hash(username_clean, passwd):
        return hashlib.sha1((username_clean + passwd).encode('utf-8')).hexdigest()

    @staticmethod
    def santatize_username(username):
        sanatized = username.replace(" ", "_")
        sanatized = sanatized.replace("'", "_")
        return sanatized.lower()

    @staticmethod
    def get_current_utc_date():
        d = datetime.utcnow()
        unixtime = calendar.timegm(d.utctimetuple())
        return unixtime

    @classmethod
    def create_group(cls, groupname):
        logger.debug(f"Creating smf group {groupname}")
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_ADD_GROUP, [groupname, groupname])
        logger.info(f"Created smf group {groupname}")
        return cls.get_group_id(groupname)

    @classmethod
    def get_group_id(cls, groupname):
        logger.debug(f"Getting smf group id for groupname {groupname}")
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_GET_GROUP_ID, [groupname])
        row = cursor.fetchone()
        logger.debug(f"Got smf group id {row[0]} for groupname {groupname}")
        return row[0]

    @classmethod
    def check_user(cls, username):
        logger.debug(f"Checking smf username {username}")
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_USER_ID_FROM_USERNAME, [cls.santatize_username(username)])
        row = cursor.fetchone()
        if row:
            logger.debug(f"Found user {username} on smf")
            return True
        logger.debug(f"User {username} not found on smf")
        return False

    @classmethod
    def add_avatar(cls, member_name, characterid):
        logger.debug(f"Adding EVE character id {characterid} portrait as smf avatar for user {member_name}")
        avatar_url = EveCharacter.generic_portrait_url(characterid, 64)
        cursor = connections['smf'].cursor()
        id_member = cls.get_user_id(member_name)
        cursor.execute(cls.SQL_ADD_USER_AVATAR, [avatar_url, id_member])

    @classmethod
    def get_user_id(cls, username):
        logger.debug(f"Getting smf user id for username {username}")
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_USER_ID_FROM_USERNAME, [username])
        row = cursor.fetchone()
        if row is not None:
            logger.debug(f"Got smf user id {row[0]} for username {username}")
            return row[0]
        else:
            logger.error(f"username {username} not found on smf. Unable to determine user id .")
            return None

    @classmethod
    def get_all_groups(cls):
        logger.debug("Getting all smf groups.")
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_GET_ALL_GROUPS)
        rows = cursor.fetchall()
        out = {}
        for row in rows:
            out[row[1]] = row[0]
        logger.debug(f"Got smf groups {out}")
        return out

    @classmethod
    def get_user_groups(cls, userid):
        logger.debug(f"Getting smf user id {userid} groups")
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_GET_USER_GROUPS, [userid])
        out = [row[0] for row in cursor.fetchall()]
        logger.debug(f"Got user {userid} smf groups {out}")
        return out

    @classmethod
    def add_user(cls, username, email_address, groups, main_character: EveCharacter) -> Tuple:
        """
        Add a user to SMF
        :param username:
        :param email_address:
        :param groups:
        :param main_character:
        :return:
        """

        main_character_id = main_character.character_id
        main_character_name = main_character.character_name

        logger.debug(
            f"Adding smf user with member_name: {username}, "
            f"email_address: {email_address}, "
            f"characterid: {main_character_id}, "
            f"main character: {main_character_name}"
        )

        cursor = connections['smf'].cursor()
        username_clean = cls.santatize_username(username)
        passwd = cls.generate_random_pass()
        pwhash = cls.gen_hash(username_clean, passwd)
        register_date = cls.get_current_utc_date()

        logger.debug(f"Proceeding to add smf user {username} and pwhash starting with {pwhash[0:5]}")

        # check if the username was simply revoked
        if cls.check_user(username) is True:
            logger.warning(
                f"Unable to add smf user with username {username} - "
                f"already exists. Updating user instead."
            )

            cls.__update_user_info(
                username_clean, email_address, pwhash, main_character_name
            )
        else:
            try:
                smf_version = cls._get_current_smf_version()
                sql_add_user_arguments = [
                    username_clean,
                    pwhash,
                    email_address,
                    register_date,
                    main_character_name,
                ]

                if version.parse(smf_version) < version.parse("2.1"):
                    logger.debug("SMF compatibility: < 2.1")

                    cursor.execute(cls.SQL_ADD_USER_SMF_20, sql_add_user_arguments)
                else:
                    logger.debug("SMF compatibility: >= 2.1")

                    cursor.execute(cls.SQL_ADD_USER_SMF_21, sql_add_user_arguments)

                cls.add_avatar(username_clean, main_character_id)
                logger.info(f"Added smf member_name {username_clean}")
                cls.update_groups(username_clean, groups)
            except Exception as e:
                logger.warning(f"Unable to add smf user {username_clean}: {e}")
                pass

        return username_clean, passwd

    @classmethod
    def __update_user_info(cls, username, email_address, passwd, main_character_name):
        logger.debug(
            f"Updating smf user {username} info: "
            f"username {email_address} "
            f"password of length {len(passwd)}"
        )
        cursor = connections['smf'].cursor()
        try:
            cursor.execute(
                cls.SQL_UPD_USER, [email_address, passwd, main_character_name, username]
            )
            logger.info(f"Updated smf user {username} info")
        except Exception as e:
            logger.exception(f"Unable to update smf user {username} info. ({e})")
            pass

    @classmethod
    def delete_user(cls, username):
        logger.debug(f"Deleting smf user {username}")
        cursor = connections['smf'].cursor()

        if cls.check_user(username):
            cursor.execute(cls.SQL_DEL_USER, [username])
            logger.info(f"Deleted smf user {username}")
            return True
        logger.error(f"Unable to delete smf user {username} - user not found on smf.")
        return False

    @classmethod
    def update_display_name(cls, user: User):
        logger.debug(f"Updating SMF displayed name for user {user}")
        cursor = connections['smf'].cursor()
        smf_username = user.smf.username

        try:
            display_name = user.profile.main_character.character_name
        except Exception as exc:
            logger.exception(
                f"Unable to find a main character name for {user}, skipping... ({exc})"
            )
            display_name = smf_username

        if cls.check_user(smf_username):
            cursor.execute(cls.SQL_UPD_DISPLAY_NAME, [display_name, smf_username])
            logger.info(f"Updated displayed name for smf user {smf_username}")
            return True
        logger.error(f"Unable to update smf user {smf_username} - user not found on smf.")
        return False

    @classmethod
    def update_groups(cls, username, groups):
        userid = cls.get_user_id(username)
        logger.debug(f"Updating smf user {username} with id {userid} groups {groups}")
        if userid is not None:
            forum_groups = cls.get_all_groups()
            user_groups = set(cls.get_user_groups(userid))
            act_groups = {cls._sanitize_groupname(g) for g in groups}
            addgroups = act_groups - user_groups
            remgroups = user_groups - act_groups
            logger.info(f"Updating smf user {username} groups - adding {addgroups}, removing {remgroups}")
            act_group_id = set()
            for g in addgroups:
                if g not in forum_groups:
                    forum_groups[g] = cls.create_group(g)
                act_group_id.add(str(cls.get_group_id(g)))
            string_groups = ','.join(act_group_id)
            cls.add_user_to_group(userid, string_groups)

    @classmethod
    def add_user_to_group(cls, userid, groupid):
        logger.debug(f"Adding smf user id {userid} to group id {groupid}")
        try:
            cursor = connections['smf'].cursor()
            cursor.execute(cls.SQL_ADD_USER_GROUP, [groupid, userid])
            logger.info(f"Added smf user id {userid} to group id {groupid}")
        except Exception as e:
            logger.exception(f"Unable to add smf user id {userid} to group id {groupid} ({e})")
            pass

    @classmethod
    def remove_user_from_group(cls, userid, groupid):
        logger.debug(f"Removing smf user id {userid} from group id {groupid}")
        try:
            cursor = connections['smf'].cursor()
            cursor.execute(cls.SQL_REMOVE_USER_GROUP, [groupid, userid])
            logger.info(f"Removed smf user id {userid} from group id {groupid}")
        except Exception as e:
            logger.exception(f"Unable to remove smf user id {userid} from group id {groupid} ({e})")
            pass

    @classmethod
    def disable_user(cls, username):
        logger.debug(f"Disabling smf user {username}")
        cursor = connections['smf'].cursor()

        password = cls.generate_random_pass()
        revoke_email = "revoked@localhost"
        try:
            pwhash = cls.gen_hash(username, password)
            cursor.execute(cls.SQL_DIS_USER, [revoke_email, pwhash, username])
            cls.get_user_id(username)
            cls.update_groups(username, [])
            logger.info(f"Disabled smf user {username}")
            return True
        except TypeError:
            logger.exception(f"TypeError occured while disabling user {username} - failed to disable.")
            return False

    @classmethod
    def update_user_password(cls, username, characterid, password=None):
        logger.debug(f"Updating smf user {username} password")
        cursor = connections['smf'].cursor()
        if not password:
            password = cls.generate_random_pass()
        if cls.check_user(username):
            username_clean = cls.santatize_username(username)
            pwhash = cls.gen_hash(username_clean, password)
            logger.debug(
                f"Proceeding to update smf user {username} "
                f"password with pwhash starting with {pwhash[0:5]}"
            )
            cursor.execute(cls.SQL_UPDATE_USER_PASSWORD, [pwhash, username])
            cls.add_avatar(username, characterid)
            logger.info(f"Updated smf user {username} password.")
            return password
        logger.error(f"Unable to update smf user {username} password - user not found on smf.")
        return ""



from django.utils.translation import gettext_lazy as _
from allianceauth.menu.hooks import MenuItemHook

from allianceauth.services.hooks import UrlHook
from allianceauth import hooks

from . import urls
from .managers import GroupManager


class GroupManagementMenuItem(MenuItemHook):
    """ This class ensures only authorized users will see the menu entry """

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            text=_("Group Management"),
            classes="fas fa-users-cog fa-fw",
            url_name="groupmanagement:management",
            order=50,
            navactive=[
                "groupmanagement:management",  # group requests view
                "groupmanagement:membership",  # group membership view
                "groupmanagement:audit_log",  # group audit log view
            ],
        )

    def render(self, request):
        if GroupManager.can_manage_groups(request.user):
            app_count = GroupManager.pending_requests_count_for_user(request.user)
            self.count = app_count if app_count and app_count > 0 else None
            return MenuItemHook.render(self, request)
        return ""


"""
            <li class="d-flex m-2 p-2 pt-0 pb-0 mt-0 mb-0">
                <i class="fas fa-users fa-fw align-self-center me-2"></i>
                <a class="nav-link flex-fill align-self-center {% navactive request 'groupmanagement:groups' %}" href="{% url 'groupmanagement:groups' %}">
                    {% translate "Groups" %}
                </a>
            </li>
"""


class GroupsMenuItem(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(
            self,
            text=_("Groups"),
            classes="fas fa-user fa-fw",
            url_name="groupmanagement:groups",
            order=25,
            navactive=[
                "groupmanagement:groups",  # group list view
            ],
        )

    def render(self, request):
        return MenuItemHook.render(self, request)


@hooks.register("menu_item_hook")
def register_manager_menu():
    return GroupManagementMenuItem()


@hooks.register("menu_item_hook")
def register_groups_menu():
    return GroupsMenuItem()


@hooks.register("url_hook")
def register_urls():
    return UrlHook(urls, "group", r"^groups/")



from django import forms
from django.contrib.auth.forms import UserChangeForm as BaseUserChangeForm
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.forms import ModelForm
from django.utils.translation import gettext_lazy as _

from allianceauth.authentication.models import User


class RegistrationForm(forms.Form):
    email = forms.EmailField(label=_('Email'), max_length=254, required=True)

    class _meta:
        model = User


class UserProfileForm(ModelForm):
    """Allows specifying FK querysets through kwarg"""

    def __init__(self, querysets=None, *args, **kwargs):
        querysets = querysets or {}
        super().__init__(*args, **kwargs)
        for field, qs in querysets.items():
            self.fields[field].queryset = qs


class UserChangeForm(BaseUserChangeForm):
    """Add custom cleaning to UserChangeForm"""

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop("request")  # Inject current request into form object
        super().__init__(*args, **kwargs)

    def clean(self):
        cleaned_data = super().clean()
        if not self.request.user.is_superuser:
            if self.instance:
                current_restricted = set(
                    self.instance.groups.filter(
                        authgroup__restricted=True
                    ).values_list("pk", flat=True)
                )
            else:
                current_restricted = set()
            new_restricted = set(
                cleaned_data["groups"].filter(
                    authgroup__restricted=True
                ).values_list("pk", flat=True)
            )
            if current_restricted != new_restricted:
                restricted_removed = current_restricted - new_restricted
                restricted_added = new_restricted - current_restricted
                restricted_changed = restricted_removed | restricted_added
                restricted_names_qs = Group.objects.filter(
                    pk__in=restricted_changed
                ).values_list("name", flat=True)
                restricted_names = ",".join(list(restricted_names_qs))
                raise ValidationError(
                    {
                        "groups": _(
                            "You are not allowed to add or remove these "
                            "restricted groups: %s" % restricted_names
                        )
                    }
                )



import logging

from django.conf import settings
from django.core.cache import cache
from django.db import models
from django.contrib.auth.models import User

logger = logging.getLogger(__name__)


class NotificationQuerySet(models.QuerySet):
    """Custom QuerySet for Notification model"""

    def update(self, *args, **kwargs):
        """Override update to ensure cache is invalidated on very call."""
        super().update(*args, **kwargs)
        user_pks = set(self.select_related("user").values_list('user__pk', flat=True))
        for user_pk in user_pks:
            NotificationManager.invalidate_user_notification_cache(user_pk)


class NotificationManager(models.Manager):

    USER_NOTIFICATION_COUNT_PREFIX = 'USER_NOTIFICATION_COUNT'
    USER_NOTIFICATION_COUNT_CACHE_DURATION = 86_400

    def get_queryset(self):
        return NotificationQuerySet(self.model, using=self._db)

    def notify_user(
        self, user: object, title: str, message: str = None, level: str = 'info'
    ) -> object:
        """Sends a new notification to user. Returns newly created notification object.
        """
        max_notifications = self._max_notifications_per_user()
        if self.filter(user=user).count() >= max_notifications:
            to_be_deleted_qs = self.filter(user=user).order_by(
                "-timestamp"
            )[max_notifications - 1:]
            for notification in to_be_deleted_qs:
                notification.delete()

        if not message:
            message = title

        if level not in self.model.Level:
            level = self.model.Level.INFO
        obj = self.create(user=user, title=title, message=message, level=level)
        logger.info("Created notification %s", obj)
        return obj

    def _max_notifications_per_user(self) -> int:
        """Maximum number of notifications allowed per user."""
        max_notifications = getattr(
            settings,
            "NOTIFICATIONS_MAX_PER_USER",
            self.model.NOTIFICATIONS_MAX_PER_USER_DEFAULT
        )
        try:
            max_notifications = int(max_notifications)
        except ValueError:
            max_notifications = None
        if max_notifications is None or max_notifications < 0:
            logger.warning(
                "NOTIFICATIONS_MAX_PER_USER setting is invalid. Using default."
            )
            max_notifications = self.model.NOTIFICATIONS_MAX_PER_USER_DEFAULT
        return max_notifications

    def user_unread_count(self, user_pk: int) -> int:
        """returns the cached unread count for a user given by user PK

        Will return -1 if user can not be found
        """
        cache_key = self._user_notification_cache_key(user_pk)
        unread_count = cache.get(key=cache_key)
        if not unread_count:
            try:
                user = User.objects.get(pk=user_pk)
            except User.DoesNotExist:
                unread_count = -1
            else:
                logger.debug(
                    'Updating notification cache for user with pk %s', user_pk
                )
                unread_count = user.notification_set.filter(viewed=False).count()
                cache.set(
                    key=cache_key,
                    value=unread_count,
                    timeout=self.USER_NOTIFICATION_COUNT_CACHE_DURATION
                )
        else:
            logger.debug(
                'Returning notification count from cache for user with pk %s', user_pk
            )

        return unread_count

    @classmethod
    def invalidate_user_notification_cache(cls, user_pk: int) -> None:
        cache.delete(key=cls._user_notification_cache_key(user_pk))
        logger.debug('Invalided notification cache for user with pk %s', user_pk)

    @classmethod
    def _user_notification_cache_key(cls, user_pk: int) -> str:
        return f'{cls.USER_NOTIFICATION_COUNT_PREFIX}_{user_pk}'



import json
from unittest.mock import patch

from django.test import RequestFactory, TestCase

from allianceauth.authentication.views import task_counts
from allianceauth.tests.auth_utils import AuthUtils

MODULE_PATH = "allianceauth.authentication.views"


def jsonresponse_to_dict(response) -> dict:
    return json.loads(response.content)


@patch(MODULE_PATH + ".queued_tasks_count")
@patch(MODULE_PATH + ".active_tasks_count")
class TestRunningTasksCount(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.factory = RequestFactory()
        cls.user = AuthUtils.create_user("bruce_wayne")

    def test_should_return_data(
        self, mock_active_tasks_count, mock_queued_tasks_count
    ):
        # given
        mock_active_tasks_count.return_value = 2
        mock_queued_tasks_count.return_value = 3
        request = self.factory.get("/")
        request.user = self.user
        # when
        response = task_counts(request)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(
            jsonresponse_to_dict(response), {"tasks_running": 2, "tasks_queued": 3}
        )



from io import StringIO

from django.core.management import call_command
from django.test import TestCase

from allianceauth.tests.auth_utils import AuthUtils

from ..models import CharacterOwnership, UserProfile


class ManagementCommandTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test user', disconnect_signals=True)
        AuthUtils.add_main_character(cls.user, 'test character', '1', '2', 'test corp', 'test')
        character = UserProfile.objects.get(user=cls.user).main_character
        CharacterOwnership.objects.create(user=cls.user, character=character, owner_hash='test')

    def setUp(self):
        self.stdout = StringIO()

    def test_ownership(self):
        call_command('checkmains', stdout=self.stdout)
        self.assertFalse(UserProfile.objects.filter(main_character__isnull=True).count())
        self.assertNotIn(self.user.username, self.stdout.getvalue())
        self.assertIn('All main characters', self.stdout.getvalue())

    def test_no_ownership(self):
        user = AuthUtils.create_user('v1 user', disconnect_signals=True)
        AuthUtils.add_main_character(user, 'v1 character', '10', '20', 'test corp', 'test')
        self.assertFalse(UserProfile.objects.filter(main_character__isnull=True).count())

        call_command('checkmains', stdout=self.stdout)
        self.assertEqual(UserProfile.objects.filter(main_character__isnull=True).count(), 1)
        self.assertIn(user.username, self.stdout.getvalue())



# Generated by Django 4.0.10 on 2023-07-16 11:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0003_menuitem_menu_menuit_rank_e880ab_idx'),
    ]

    operations = [
        migrations.AlterField(
            model_name='menuitem',
            name='hide',
            field=models.BooleanField(default=False, help_text='Hide this menu item. If this item is a header all items under it will be hidden too.'),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='icon_classes',
            field=models.CharField(blank=True, default=None, help_text='Font Awesome classes to show as icon on menu', max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='parent',
            field=models.ForeignKey(blank=True, help_text='Parent Header. (Optional)', null=True, on_delete=django.db.models.deletion.SET_NULL, to='menu.menuitem'),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='rank',
            field=models.IntegerField(default=1000, help_text='Order of the menu. Lowest First.'),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='text',
            field=models.CharField(blank=True, default=None, help_text='Text to show on menu', max_length=150, null=True),
        ),
    ]



# Generated by Django 2.2.11 on 2020-05-22 13:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0009_set_mumble_dissplay_names'),
    ]

    operations = [
        migrations.AddField(
            model_name='mumbleuser',
            name='certhash',
            field=models.CharField(blank=True, editable=False, help_text='Hash of Mumble client certificate as presented when user authenticates', max_length=254, null=True, verbose_name='Certificate Hash'),
        ),
    ]



from django.apps import AppConfig
from django.core.checks import Warning, Error, register


class AllianceAuthConfig(AppConfig):
    name = 'allianceauth'


@register()
def check_settings(app_configs, **kwargs):
    from django.conf import settings

    errors = []
    if hasattr(settings, "SITE_URL"):
        if settings.SITE_URL[-1] == "/":
            errors.append(Warning(
                "'SITE_URL' Has a trailing slash. This may lead to incorrect links being generated by Auth."))
    else:
        errors.append(Error(
            "No 'SITE_URL' found is settings. This may lead to incorrect links being generated by Auth or Errors in 3rd party modules."))
    if hasattr(settings, "CSRF_TRUSTED_ORIGINS"):
        if hasattr(settings, "SITE_URL"):
            if settings.SITE_URL not in settings.CSRF_TRUSTED_ORIGINS:
                errors.append(Warning(
                    "'SITE_URL' not found in 'CSRF_TRUSTED_ORIGINS'. Auth may not load pages correctly until this is rectified."))
    else:
        errors.append(Error(
            "No 'CSRF_TRUSTED_ORIGINS' found is settings, Auth may not load pages correctly until this is rectified"))

    return errors



# Generated by Django 4.0.6 on 2022-07-18 17:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('timerboard', '0004_timer_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timer',
            name='planet_moon',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
    ]



# Generated by Django 1.11.5 on 2017-09-28 02:16

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('timerboard', '0002_make_strings_more_stringy'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timer',
            name='eve_character',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='eveonline.EveCharacter'),
        ),
        migrations.AlterField(
            model_name='timer',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
    ]



import logging
import os

from django.conf import settings


logger = logging.getLogger(__name__)


class LoggerAddTag(logging.LoggerAdapter):
    """add custom tag to a logger"""
    def __init__(self, logger, prefix):
        super().__init__(logger, {})
        self.prefix = prefix

    def process(self, msg, kwargs):
        return f'[{self.prefix}] {msg}', kwargs


def clean_setting(
    name: str,
    default_value: object,
    min_value: int = None,
    max_value: int = None,
    required_type: type = None
):
    """cleans the input for a custom setting

    Will use `default_value` if settings does not exit or has the wrong type
    or is outside define boundaries (for int only)

    Need to define `required_type` if `default_value` is `None`

    Will assume `min_value` of 0 for int (can be overriden)

    Returns cleaned value for setting
    """
    if default_value is None and not required_type:
        raise ValueError('You must specify a required_type for None defaults')

    if not required_type:
        required_type = type(default_value)

    if min_value is None and required_type == int:
        min_value = 0

    if not hasattr(settings, name):
        cleaned_value = default_value
    else:
        if (
            isinstance(getattr(settings, name), required_type)
            and (min_value is None or getattr(settings, name) >= min_value)
            and (max_value is None or getattr(settings, name) <= max_value)
        ):
            cleaned_value = getattr(settings, name)
        else:
            logger.warning(
                'You setting for %s it not valid. Please correct it. '
                'Using default for now: %s',
                name,
                default_value
            )
            cleaned_value = default_value
    return cleaned_value


def set_logger_to_file(logger_name: str, name: str) -> object:
    """set logger for current module to log into a file. Useful for tests.

    Args:
    - logger: current logger object
    - name: name of current module, e.g. __file__

    Returns:
    - amended logger
    """

    # reconfigure logger so we get logging from tested module
    f_format = logging.Formatter(
        '%(asctime)s - %(levelname)s - %(module)s:%(funcName)s - %(message)s'
    )
    path = os.path.splitext(name)[0]
    f_handler = logging.FileHandler(f'{path}.log', 'w+')
    f_handler.setFormatter(f_format)
    logger = logging.getLogger(logger_name)
    logger.level = logging.DEBUG
    logger.addHandler(f_handler)
    logger.propagate = False
    return logger



import logging
from django.db import models, transaction
from django.contrib.auth.models import Group, User
from django.core.exceptions import ObjectDoesNotExist

from allianceauth.authentication.models import State
from allianceauth.eveonline.models import EveCorporationInfo, EveAllianceInfo

logger = logging.getLogger(__name__)


def get_users_for_state(state: State):
    return User.objects.select_related('profile').prefetch_related('profile__main_character')\
            .filter(profile__state_id=state.pk)


class AutogroupsConfigManager(models.Manager):
    def update_groups_for_state(self, state: State):
        """
        Update all the Group memberships for the users
        who have State
        :param state: State to update for
        :return:
        """
        users = get_users_for_state(state)
        for config in self.filter(states=state):
            logger.debug("in state loop")
            for user in users:
                logger.debug(f"in user loop for {user}")
                config.update_group_membership_for_user(user)

    def update_groups_for_user(self, user: User, state: State = None):
        """
        Update the Group memberships for the given users state
        :param user: User to update for
        :param state: State to update user for
        :return:
        """
        if state is None:
            state = user.profile.state
        for config in self.filter(states=state):
            # grant user new groups for their state
            config.update_group_membership_for_user(user)
        for config in self.exclude(states=state):
            # ensure user does not have groups from previous state
            config.remove_user_from_alliance_groups(user)
            config.remove_user_from_corp_groups(user)


class AutogroupsConfig(models.Model):
    OPT_TICKER = 'ticker'
    OPT_NAME = 'name'
    NAME_OPTIONS = (
        (OPT_TICKER, 'Ticker'),
        (OPT_NAME, 'Full name'),
    )

    states = models.ManyToManyField(State, related_name='autogroups')

    corp_groups = models.BooleanField(default=False, help_text="Setting this to false will delete all the created groups.")
    corp_group_prefix = models.CharField(max_length=50, default='Corp ', blank=True)
    corp_name_source = models.CharField(max_length=20, choices=NAME_OPTIONS, default=OPT_NAME)

    alliance_groups = models.BooleanField(default=False, help_text="Setting this to false will delete all the created groups.")
    alliance_group_prefix = models.CharField(max_length=50, default='Alliance ', blank=True)
    alliance_name_source = models.CharField(max_length=20, choices=NAME_OPTIONS, default=OPT_NAME)

    corp_managed_groups = models.ManyToManyField(
        Group, through='ManagedCorpGroup', related_name='corp_managed_config',
        help_text='A list of corporation groups created and maintained by this AutogroupConfig. You should not edit this list unless you know what you\'re doing.')

    alliance_managed_groups = models.ManyToManyField(
        Group, through='ManagedAllianceGroup', related_name='alliance_managed_config',
        help_text='A list of alliance groups created and maintained by this AutogroupConfig. You should not edit this list unless you know what you\'re doing.')

    replace_spaces = models.BooleanField(default=False)
    replace_spaces_with = models.CharField(
        max_length=10, default='', blank=True,
        help_text='Any spaces in the group name will be replaced with this.')

    objects = AutogroupsConfigManager()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return 'States: ' + (' '.join(list(self.states.all().values_list('name', flat=True))) if self.pk else str(None))

    def update_all_states_group_membership(self):
        list(map(self.update_group_membership_for_state, self.states.all()))

    def update_group_membership_for_state(self, state: State):
        list(map(self.update_group_membership_for_user, get_users_for_state(state)))

    @transaction.atomic
    def update_group_membership_for_user(self, user: User):
        self.update_alliance_group_membership(user)
        self.update_corp_group_membership(user)

    def user_entitled_to_groups(self, user: User) -> bool:
        try:
            return user.profile.state in self.states.all()
        except ObjectDoesNotExist:
            return False

    @transaction.atomic
    def update_alliance_group_membership(self, user: User):
        group = None
        try:
            if not self.alliance_groups or not self.user_entitled_to_groups(user):
                logger.debug(f'User {user} does not have required state for alliance group membership')
                return
            else:
                alliance = user.profile.main_character.alliance
                if alliance is None:
                    logger.debug(f'User {user} alliance is None, cannot update group membership')
                    return
                group = self.get_alliance_group(alliance)
        except EveAllianceInfo.DoesNotExist:
            logger.debug(f'User {user} main characters alliance does not exist in the database. Creating.')
            alliance = EveAllianceInfo.objects.create_alliance(user.profile.main_character.alliance_id)
            group = self.get_alliance_group(alliance)
        except AttributeError:
            logger.warning(f'User {user} does not have a main character. Group membership not updated')
        finally:
            self.remove_user_from_alliance_groups(user, except_group=group)
            if group is not None:
                logger.debug(f'Adding user {user} to alliance group {group}')
                user.groups.add(group)

    @transaction.atomic
    def update_corp_group_membership(self, user: User):
        group = None
        try:
            if not self.corp_groups or not self.user_entitled_to_groups(user):
                logger.debug(f'User {user} does not have required state for corp group membership')
            else:
                corp = user.profile.main_character.corporation
                group = self.get_corp_group(corp)
        except EveCorporationInfo.DoesNotExist:
            logger.debug(f'User {user} main characters corporation does not exist in the database. Creating.')
            corp = EveCorporationInfo.objects.create_corporation(user.profile.main_character.corporation_id)
            group = self.get_corp_group(corp)
        except AttributeError:
            logger.warning(f'User {user} does not have a main character. Group membership not updated')
        finally:
            self.remove_user_from_corp_groups(user, except_group=group)
            if group is not None:
                logger.debug(f'Adding user {user} to corp group {group}')
                user.groups.add(group)

    @transaction.atomic
    def remove_user_from_alliance_groups(self, user: User, except_group: Group = None):
        remove_groups = user.groups.filter(pk__in=self.alliance_managed_groups.all().values_list('pk', flat=True))
        if except_group is not None:
            remove_groups = remove_groups.exclude(pk=except_group.pk)
        list(map(user.groups.remove, remove_groups))

    @transaction.atomic
    def remove_user_from_corp_groups(self, user: User, except_group: Group = None):
        remove_groups = user.groups.filter(pk__in=self.corp_managed_groups.all().values_list('pk', flat=True))
        if except_group is not None:
            remove_groups = remove_groups.exclude(pk=except_group.pk)
        list(map(user.groups.remove, remove_groups))

    def get_alliance_group(self, alliance: EveAllianceInfo) -> Group:
        return self.create_alliance_group(alliance)

    def get_corp_group(self, corp: EveCorporationInfo) -> Group:
        return self.create_corp_group(corp)

    @transaction.atomic
    def create_alliance_group(self, alliance: EveAllianceInfo) -> Group:
        group, created = Group.objects.get_or_create(name=self.get_alliance_group_name(alliance))
        ManagedAllianceGroup.objects.get_or_create(group=group, config=self, alliance=alliance)
        return group

    @transaction.atomic
    def create_corp_group(self, corp: EveCorporationInfo) -> Group:
        group, created = Group.objects.get_or_create(name=self.get_corp_group_name(corp))
        ManagedCorpGroup.objects.get_or_create(group=group, config=self, corp=corp)
        return group

    def delete_alliance_managed_groups(self):
        """
        Deletes ALL managed alliance groups
        """
        for g in self.alliance_managed_groups.all():
            g.delete()

    def delete_corp_managed_groups(self):
        """
        Deletes ALL managed corp groups
        """
        for g in self.corp_managed_groups.all():
            g.delete()

    def get_alliance_group_name(self, alliance: EveAllianceInfo) -> str:
        if self.alliance_name_source == self.OPT_TICKER:
            name = alliance.alliance_ticker
        elif self.alliance_name_source == self.OPT_NAME:
            name = alliance.alliance_name
        else:
            raise NameSourceException('Not a valid name source')
        return self._replace_spaces(self.alliance_group_prefix + name)

    def get_corp_group_name(self, corp: EveCorporationInfo) -> str:
        if self.corp_name_source == self.OPT_TICKER:
            name = corp.corporation_ticker
        elif self.corp_name_source == self.OPT_NAME:
            name = corp.corporation_name
        else:
            raise NameSourceException('Not a valid name source')
        return self._replace_spaces(self.corp_group_prefix + name)

    def _replace_spaces(self, name: str) -> str:
        """
        Replace the spaces in the given name based on the config
        :param name: name to replace spaces in
        :return: name with spaces replaced with the configured character(s) or unchanged if configured
        """
        if self.replace_spaces:
            return name.strip().replace(' ', str(self.replace_spaces_with))
        return name


class ManagedGroup(models.Model):
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    config = models.ForeignKey(AutogroupsConfig, on_delete=models.CASCADE)

    class Meta:
        abstract = True

    def __str__(self):
        return "Managed Group: %s" % self.group.name

class ManagedCorpGroup(ManagedGroup):
    corp = models.ForeignKey(EveCorporationInfo, on_delete=models.CASCADE)


class ManagedAllianceGroup(ManagedGroup):
    alliance = models.ForeignKey(EveAllianceInfo, on_delete=models.CASCADE)


class NameSourceException(Exception):
    pass



from unittest.mock import Mock, patch

from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase
from esi.models import Token

from allianceauth.tests.auth_utils import AuthUtils

from ..evelinks import eveimageserver
from ..models import EveAllianceInfo, EveCharacter, EveCorporationInfo, EveFactionInfo
from ..providers import Alliance, Character, Corporation
from .esi_client_stub import EsiClientStub


class EveCharacterTestCase(TestCase):
    def test_corporation_prop(self):
        """
        Test that the correct corporation is returned by the corporation property
        """
        character = EveCharacter.objects.create(
            character_id=1234,
            character_name='character.name',
            corporation_id=2345,
            corporation_name='character.corp.name',
            corporation_ticker='cc1',
            alliance_id=12345,
            alliance_name='character.alliance.name',
        )

        expected = EveCorporationInfo.objects.create(
            corporation_id=2345,
            corporation_name='corp.name',
            corporation_ticker='cc1',
            member_count=10,
            alliance=None,
        )

        incorrect = EveCorporationInfo.objects.create(
            corporation_id=9999,
            corporation_name='corp.name1',
            corporation_ticker='cc11',
            member_count=10,
            alliance=None,
        )

        self.assertEqual(character.corporation, expected)
        self.assertNotEqual(character.corporation, incorrect)

    def test_corporation_prop_exception(self):
        """
        Check that an exception is raised when the expected
        object is not in the database
        """
        character = EveCharacter.objects.create(
            character_id=1234,
            character_name='character.name',
            corporation_id=2345,
            corporation_name='character.corp.name',
            corporation_ticker='cc1',
            alliance_id=123456,
            alliance_name='character.alliance.name',
        )

        with self.assertRaises(EveCorporationInfo.DoesNotExist):
            character.corporation

    def test_alliance_prop(self):
        """
        Test that the correct alliance is returned by the alliance property
        """
        character = EveCharacter.objects.create(
            character_id=1234,
            character_name='character.name',
            corporation_id=2345,
            corporation_name='character.corp.name',
            corporation_ticker='cc1',
            alliance_id=3456,
            alliance_name='character.alliance.name',
        )

        expected = EveAllianceInfo.objects.create(
            alliance_id=3456,
            alliance_name='alliance.name',
            alliance_ticker='ac2',
            executor_corp_id=2345,
        )

        incorrect = EveAllianceInfo.objects.create(
            alliance_id=9001,
            alliance_name='alliance.name1',
            alliance_ticker='ac1',
            executor_corp_id=2654,
        )

        self.assertEqual(character.alliance, expected)
        self.assertNotEqual(character.alliance, incorrect)

    def test_alliance_prop_exception(self):
        """
        Check that an exception is raised when the expected
        object is not in the database
        """
        character = EveCharacter.objects.create(
            character_id=1234,
            character_name='character.name',
            corporation_id=2345,
            corporation_name='character.corp.name',
            corporation_ticker='cc1',
            alliance_id=3456,
            alliance_name='character.alliance.name',
        )

        with self.assertRaises(EveAllianceInfo.DoesNotExist):
            character.alliance

    def test_alliance_prop_none(self):
        """
        Check that None is returned when the character has no alliance
        """
        character = EveCharacter.objects.create(
            character_id=1234,
            character_name='character.name',
            corporation_id=2345,
            corporation_name='character.corp.name',
            corporation_ticker='cc1',
            alliance_id=None,
            alliance_name=None,
        )

        self.assertIsNone(character.alliance)

    def test_faction_prop(self):
        """
        Test that the correct faction is returned by the alliance property
        """
        character = EveCharacter.objects.create(
            character_id=1234,
            character_name='character.name',
            corporation_id=2345,
            corporation_name='character.corp.name',
            corporation_ticker='cc1',
            alliance_id=3456,
            alliance_name='character.alliance.name',
            faction_id=1337,
            faction_name='character.faction.name'
        )

        expected = EveFactionInfo.objects.create(faction_id=1337, faction_name='faction.name')
        incorrect = EveFactionInfo.objects.create(faction_id=8008, faction_name='faction.badname')

        self.assertEqual(character.faction, expected)
        self.assertNotEqual(character.faction, incorrect)

    def test_faction_prop_exception(self):
        """
        Check that an exception is raised when the expected
        object is not in the database
        """
        character = EveCharacter.objects.create(
            character_id=1234,
            character_name='character.name',
            corporation_id=2345,
            corporation_name='character.corp.name',
            corporation_ticker='cc1',
            alliance_id=3456,
            alliance_name='character.alliance.name',
            faction_id=1337,
            faction_name='character.faction.name'
        )

        with self.assertRaises(EveFactionInfo.DoesNotExist):
            character.faction

    def test_faction_prop_none(self):
        """
        Check that None is returned when the character has no alliance
        """
        character = EveCharacter.objects.create(
            character_id=1234,
            character_name='character.name',
            corporation_id=2345,
            corporation_name='character.corp.name',
            corporation_ticker='cc1',
            alliance_id=None,
            alliance_name=None,
            faction_id=None,
            faction_name=None,
        )

        self.assertIsNone(character.faction)

    @patch('allianceauth.eveonline.providers.provider')
    def test_update_character(self, mock_provider):
        mock_provider.get_corp.return_value = Corporation(
            id=2002,
            name='Dummy Corp 2',
            ticker='DC2',
            ceo_id=1001,
            members=34,
        )

        my_character = EveCharacter.objects.create(
            character_id=1001,
            character_name='Bruce Wayne',
            corporation_id=2001,
            corporation_name='Dummy Corp 1',
            corporation_ticker='DC1',
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            faction_id=1337,
            faction_name='Dummy Faction 1',
        )
        my_updated_character = Character(
            name='Bruce X. Wayne',
            corp_id=2002,
            faction_id=None,
        )
        my_character.update_character(my_updated_character)
        self.assertEqual(my_character.character_name, 'Bruce X. Wayne')
        self.assertFalse(my_character.faction_id)

        # todo: add test cases not yet covered, e.g. with alliance

    def test_image_url(self):
        self.assertEqual(
            EveCharacter.generic_portrait_url(42),
            eveimageserver._eve_entity_image_url('character', 42)
        )
        self.assertEqual(
            EveCharacter.generic_portrait_url(42, 256),
            eveimageserver._eve_entity_image_url('character', 42, 256)
        )

    def test_portrait_urls(self):
        x = EveCharacter(
            character_id=42,
            character_name='character.name',
            corporation_id=123,
            corporation_name='corporation.name',
            corporation_ticker='ABC',
        )
        self.assertEqual(
            x.portrait_url(),
            eveimageserver._eve_entity_image_url('character', 42)
        )
        self.assertEqual(
            x.portrait_url(64),
            eveimageserver._eve_entity_image_url('character', 42, size=64)
        )
        self.assertEqual(
            x.portrait_url_32,
            eveimageserver._eve_entity_image_url('character', 42, size=32)
        )
        self.assertEqual(
            x.portrait_url_64,
            eveimageserver._eve_entity_image_url('character', 42, size=64)
        )
        self.assertEqual(
            x.portrait_url_128,
            eveimageserver._eve_entity_image_url('character', 42, size=128)
        )
        self.assertEqual(
            x.portrait_url_256,
            eveimageserver._eve_entity_image_url('character', 42, size=256)
        )

    def test_corporation_logo_urls(self):
        x = EveCharacter(
            character_id=42,
            character_name='character.name',
            corporation_id=123,
            corporation_name='corporation.name',
            corporation_ticker='ABC',
        )
        self.assertEqual(
            x.corporation_logo_url(),
            eveimageserver._eve_entity_image_url('corporation', 123)
        )
        self.assertEqual(
            x.corporation_logo_url(256),
            eveimageserver._eve_entity_image_url('corporation', 123, size=256)
        )
        self.assertEqual(
            x.corporation_logo_url_32,
            eveimageserver._eve_entity_image_url('corporation', 123, size=32)
        )
        self.assertEqual(
            x.corporation_logo_url_64,
            eveimageserver._eve_entity_image_url('corporation', 123, size=64)
        )
        self.assertEqual(
            x.corporation_logo_url_128,
            eveimageserver._eve_entity_image_url('corporation', 123, size=128)
        )
        self.assertEqual(
            x.corporation_logo_url_256,
            eveimageserver._eve_entity_image_url('corporation', 123, size=256)
        )

    def test_alliance_logo_urls(self):
        x = EveCharacter(
            character_id=42,
            character_name='character.name',
            corporation_id=123,
            corporation_name='corporation.name',
            corporation_ticker='ABC',
        )
        self.assertEqual(
            x.alliance_logo_url(),
            ''
        )
        self.assertEqual(
            x.alliance_logo_url_32,
            ''
        )
        self.assertEqual(
            x.alliance_logo_url_64,
            ''
        )
        self.assertEqual(
            x.alliance_logo_url_128,
            ''
        )
        self.assertEqual(
            x.alliance_logo_url_256,
            ''
        )
        x.alliance_id = 987
        self.assertEqual(
            x.alliance_logo_url(),
            eveimageserver._eve_entity_image_url('alliance', 987)
        )
        self.assertEqual(
            x.alliance_logo_url(128),
            eveimageserver._eve_entity_image_url('alliance', 987, size=128)
        )
        self.assertEqual(
            x.alliance_logo_url_32,
            eveimageserver._eve_entity_image_url('alliance', 987, size=32)
        )
        self.assertEqual(
            x.alliance_logo_url_64,
            eveimageserver._eve_entity_image_url('alliance', 987, size=64)
        )
        self.assertEqual(
            x.alliance_logo_url_128,
            eveimageserver._eve_entity_image_url('alliance', 987, size=128)
        )
        self.assertEqual(
            x.alliance_logo_url_256,
            eveimageserver._eve_entity_image_url('alliance', 987, size=256)
        )


class EveAllianceTestCase(TestCase):

    def test_str(self):
        my_alliance = EveAllianceInfo(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        self.assertEqual(str(my_alliance), 'Dummy Alliance 1')

    @patch(
        'allianceauth.eveonline.models.EveCorporationInfo.objects.create_corporation'
    )
    def test_populate_alliance(self, mock_create_corporation):

        def create_corp(corp_id):
            if corp_id == 2002:
                EveCorporationInfo.objects.create(
                    corporation_id=2002,
                    corporation_name='Dummy Corporation 2',
                    corporation_ticker='DC2',
                    member_count=87,
                )
            else:
                raise ValueError()

        mock_EveAllianceProviderManager = Mock()
        mock_EveAllianceProviderManager.get_alliance.return_value = \
            Alliance(
                id=3001,
                name='Dummy Alliance 1',
                corp_ids=[2001, 2002]
            )
        mock_create_corporation.side_effect = create_corp

        EveCorporationInfo.objects.create(
            corporation_id=2001,
            corporation_name='Dummy Corporation 1',
            corporation_ticker='DC1',
            member_count=42,
        )

        my_alliance = EveAllianceInfo(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        my_alliance.provider = mock_EveAllianceProviderManager
        my_alliance.save()
        my_alliance.populate_alliance()

        for corporation in (
            EveCorporationInfo.objects.filter(corporation_id__in=[2001, 2002])
        ):
            self.assertEqual(corporation.alliance, my_alliance)

    def test_update_alliance_with_object(self):
        my_alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        updated_alliance = Alliance(
            id=3002,
            name='Dummy Alliance 2',
            corp_ids=[2004],
            executor_corp_id=2004
        )
        my_alliance.update_alliance(updated_alliance)
        my_alliance.refresh_from_db()
        self.assertEqual(int(my_alliance.executor_corp_id), 2004)

        # potential bug
        # update_alliance() is only updateting executor_corp_id when object is given

    def test_update_alliance_wo_object(self):
        mock_EveAllianceProviderManager = Mock()
        mock_EveAllianceProviderManager.get_alliance.return_value = \
            Alliance(
                id=3002,
                name='Dummy Alliance 2',
                corp_ids=[2004],
                executor_corp_id=2004
            )

        my_alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        my_alliance.provider = mock_EveAllianceProviderManager
        my_alliance.save()
        Alliance(
            name='Dummy Alliance 2',
            corp_ids=[2004],
            executor_corp_id=2004
        )
        my_alliance.update_alliance()
        my_alliance.refresh_from_db()
        self.assertEqual(int(my_alliance.executor_corp_id), 2004)

        # potential bug
        # update_alliance() is only updateting executor_corp_id nothing else ???

    def test_image_url(self):
        self.assertEqual(
            EveAllianceInfo.generic_logo_url(42),
            eveimageserver._eve_entity_image_url('alliance', 42)
        )
        self.assertEqual(
            EveAllianceInfo.generic_logo_url(42, 256),
            eveimageserver._eve_entity_image_url('alliance', 42, 256)
        )

    def test_logo_url(self):
        x = EveAllianceInfo(
            alliance_id=42,
            alliance_name='alliance.name',
            alliance_ticker='ABC',
            executor_corp_id=123
        )
        self.assertEqual(
            x.logo_url(),
            'https://images.evetech.net/alliances/42/logo?size=32'
        )
        self.assertEqual(
            x.logo_url(64),
            'https://images.evetech.net/alliances/42/logo?size=64'
        )
        self.assertEqual(
            x.logo_url_32,
            'https://images.evetech.net/alliances/42/logo?size=32'
        )
        self.assertEqual(
            x.logo_url_64,
            'https://images.evetech.net/alliances/42/logo?size=64'
        )
        self.assertEqual(
            x.logo_url_128,
            'https://images.evetech.net/alliances/42/logo?size=128'
        )
        self.assertEqual(
            x.logo_url_256,
            'https://images.evetech.net/alliances/42/logo?size=256'
        )


class EveCorporationTestCase(TestCase):

    def setUp(self):
        my_alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        self.my_corp = EveCorporationInfo(
            corporation_id=2001,
            corporation_name='Dummy Corporation 1',
            corporation_ticker='DC1',
            member_count=42,
            alliance=my_alliance
        )

    def test_str(self):
        self.assertEqual(str(self.my_corp), 'Dummy Corporation 1')

    def test_update_corporation_from_object_w_alliance(self):
        updated_corp = Corporation(
            members=87
        )
        self.my_corp.update_corporation(updated_corp)
        self.assertEqual(self.my_corp.member_count, 87)

        # potential bug
        # update_corporation updates member_count only

    def test_update_corporation_no_object_w_alliance(self):
        mock_provider = Mock()
        mock_provider.get_corporation.return_value = Corporation(members=87)
        self.my_corp.provider = mock_provider

        self.my_corp.update_corporation()
        self.assertEqual(self.my_corp.member_count, 87)

    def test_update_corporation_from_object_wo_alliance(self):
        my_corp2 = EveCorporationInfo(
            corporation_id=2011,
            corporation_name='Dummy Corporation 11',
            corporation_ticker='DC11',
            member_count=6
        )
        updated_corp = Corporation(
            members=8
        )
        my_corp2.update_corporation(updated_corp)
        self.assertEqual(my_corp2.member_count, 8)
        self.assertIsNone(my_corp2.alliance)

    def test_image_url(self):
        self.assertEqual(
            EveCorporationInfo.generic_logo_url(42),
            eveimageserver._eve_entity_image_url('corporation', 42)
        )
        self.assertEqual(
            EveCorporationInfo.generic_logo_url(42, 256),
            eveimageserver._eve_entity_image_url('corporation', 42, 256)
        )

    def test_logo_url(self):
        self.assertEqual(
            self.my_corp.logo_url(),
            'https://images.evetech.net/corporations/2001/logo?size=32'
        )
        self.assertEqual(
            self.my_corp.logo_url(64),
            'https://images.evetech.net/corporations/2001/logo?size=64'
        )
        self.assertEqual(
            self.my_corp.logo_url_32,
            'https://images.evetech.net/corporations/2001/logo?size=32'
        )
        self.assertEqual(
            self.my_corp.logo_url_64,
            'https://images.evetech.net/corporations/2001/logo?size=64'
        )
        self.assertEqual(
            self.my_corp.logo_url_128,
            'https://images.evetech.net/corporations/2001/logo?size=128'
        )
        self.assertEqual(
            self.my_corp.logo_url_256,
            'https://images.evetech.net/corporations/2001/logo?size=256'
        )


@patch('allianceauth.eveonline.providers.esi_client_factory')
@patch("allianceauth.eveonline.models.notify")
class TestCharacterUpdate(TestCase):
    def test_should_update_normal_character(self, mock_notify, mock_esi_client_factory):
        # given
        mock_esi_client_factory.return_value = EsiClientStub()
        my_character = EveCharacter.objects.create(
            character_id=1001,
            character_name="not my name",
            corporation_id=2002,
            corporation_name="Wayne Food",
            corporation_ticker="WYF",
            alliance_id=None
        )
        # when
        my_character.update_character()
        # then
        my_character.refresh_from_db()
        self.assertEqual(my_character.character_name, "Bruce Wayne")
        self.assertEqual(my_character.corporation_id, 2001)
        self.assertEqual(my_character.corporation_name, "Wayne Technologies")
        self.assertEqual(my_character.corporation_ticker, "WTE")
        self.assertEqual(my_character.alliance_id, 3001)
        self.assertEqual(my_character.alliance_name, "Wayne Enterprises")
        self.assertEqual(my_character.alliance_ticker, "WYE")
        self.assertFalse(mock_notify.called)

    def test_should_update_dead_character_with_owner(
        self, mock_notify, mock_esi_client_factory
    ):
        # given
        mock_esi_client_factory.return_value = EsiClientStub()
        character_1666 = EveCharacter.objects.create(
            character_id=1666,
            character_name="Hal Jordan",
            corporation_id=2002,
            corporation_name="Wayne Food",
            corporation_ticker="WYF",
            alliance_id=None
        )
        user = AuthUtils.create_user("Bruce Wayne")
        token_1666 = Token.objects.create(
            user=user,
            character_id=character_1666.character_id,
            character_name=character_1666.character_name,
            character_owner_hash="ABC123-1666",
        )
        character_1001 = EveCharacter.objects.create(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2001,
            corporation_name="Wayne Technologies",
            corporation_ticker="WYT",
            alliance_id=None
        )
        token_1001 = Token.objects.create(
            user=user,
            character_id=character_1001.character_id,
            character_name=character_1001.character_name,
            character_owner_hash="ABC123-1001",
        )
        # when
        character_1666.update_character()
        # then
        character_1666.refresh_from_db()
        self.assertTrue(character_1666.is_biomassed)
        self.assertNotIn(token_1666, user.token_set.all())
        self.assertIn(token_1001, user.token_set.all())
        with self.assertRaises(ObjectDoesNotExist):
            self.assertTrue(character_1666.character_ownership)
        user.profile.refresh_from_db()
        self.assertIsNone(user.profile.main_character)
        self.assertTrue(mock_notify.called)

    def test_should_handle_dead_character_without_owner(
        self, mock_notify, mock_esi_client_factory
    ):
        # given
        mock_esi_client_factory.return_value = EsiClientStub()
        character_1666 = EveCharacter.objects.create(
            character_id=1666,
            character_name="Hal Jordan",
            corporation_id=1011,
            corporation_name="LexCorp",
            corporation_ticker='LC',
            alliance_id=None
        )
        # when
        character_1666.update_character()
        # then
        character_1666.refresh_from_db()
        self.assertTrue(character_1666.is_biomassed)
        self.assertFalse(mock_notify.called)



import warnings
from django.views.generic.base import View
from django.http import HttpResponseRedirect
from django.shortcuts import render, redirect
from django.contrib import messages
from django.conf import settings
import logging

logger = logging.getLogger(__name__)


class NightModeRedirectView(View):
    SESSION_VAR = "NIGHT_MODE"

    def get(self, request, *args, **kwargs):
        request.session[self.SESSION_VAR] = not self.night_mode_state(request)
        if not request.user.is_anonymous:
            try:
                request.user.profile.night_mode = request.session[self.SESSION_VAR]
                request.user.profile.save()
            except Exception as e:
                logger.exception(e)

        return HttpResponseRedirect(request.GET.get("next", "/"))

    @classmethod
    def night_mode_state(cls, request):
        try:
            return request.session.get(cls.SESSION_VAR, False)
        except AttributeError:
            # Session is middleware
            # Sometimes request wont have a session attribute
            return False


class ThemeRedirectView(View):
    THEME_VAR = "THEME"

    def post(self, request, *args, **kwargs):
        theme = request.POST.get('theme', settings.DEFAULT_THEME)
        if not request.user.is_anonymous:
            try:
                request.user.profile.theme = theme
                request.user.profile.save()
                request.session[self.THEME_VAR] = theme
            except Exception as e:
                logger.exception(e)

        return HttpResponseRedirect(request.GET.get("next", "/"))


def Generic500Redirect(request):  # TODO Real view

    title = _(
        "Internal Server Error"
    )
    message = _(
        "Auth encountered an error processing your request, please try again. "
        "If the error persists, please contact the administrators."
    )

    return render(request, "allianceauth/error.html", context={"error_title": title, "error_message": message})


def Generic404Redirect(request, exception):  # TODO Real view
    title = _(
        "Page Not Found"
    )
    message = _(
        "Page does not exist. If you believe this is in error please contact the administrators. "
    )

    return render(request, "allianceauth/error.html", context={"error_title": title, "error_message": message})


def Generic403Redirect(request, exception):  # TODO Real view
    title = _(
        "Permission Denied"
    )
    message = _(
        "You do not have permission to access the requested page. "
        "If you believe this is in error please contact the administrators."
    )

    return render(request, "allianceauth/error.html", context={"error_title": title, "error_message": message})


def Generic400Redirect(request, exception):  # TODO Real view
    title = _(
        "Bad Request"
    )
    message = _(
        "Auth encountered an error processing your request, please try again. "
        "If the error persists, please contact the administrators."
    )

    return render(request, "allianceauth/error.html", context={"error_title": title, "error_message": message})



# Generated by Django 3.2.8 on 2021-10-26 16:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("optimer", "0004_on_delete"),
    ]

    operations = [
        migrations.CreateModel(
            name="OpTimerType",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("type", models.CharField(default="", max_length=254)),
            ],
            options={
                "ordering": ["type"],
                "default_permissions": (),
            },
        ),
        migrations.AlterModelOptions(
            name="optimer",
            options={"default_permissions": (), "ordering": ["start"]},
        ),
        migrations.AddField(
            model_name="optimer",
            name="description",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AddField(
            model_name="optimer",
            name="type",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="optimer.optimertype",
            ),
        ),
    ]



from django.contrib import admin
from .models import Phpbb3User
from ...admin import ServicesUserAdmin


@admin.register(Phpbb3User)
class Phpbb3UserAdmin(ServicesUserAdmin):
    list_display = ServicesUserAdmin.list_display + ('username',)
    search_fields = ServicesUserAdmin.search_fields + ('username', )



from django.contrib.auth.models import User
from django.core.management.base import BaseCommand

from allianceauth.services.tasks import validate_services


class Command(BaseCommand):
    help = "Ensures all service accounts belong to users with required permissions."

    def handle(self, *args, **options):
        for u in User.objects.all():
            validate_services(u.pk)
        self.stdout.write(self.style.SUCCESS('Verified all user service accounts.'))



from django.db import migrations
from ..manager import SmfManager

def on_migrate(apps, schema_editor):
    SmfUser = apps.get_model("smf", "SmfUser")
    db_alias = schema_editor.connection.alias
    all_smf_users = SmfUser.objects.using(db_alias).all()

    for smf_user in all_smf_users:
        try:
            auth_user = smf_user.user
        except:
            pass
        else:
            SmfManager.update_display_name(auth_user)

def on_migrate_zero(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('smf', '0002_service_permissions'),
    ]

    operations = [
        migrations.RunPython(on_migrate, on_migrate_zero),
    ]



# Generated by Django 4.0.2 on 2022-08-28 14:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='menuitem',
            name='hook_function',
            field=models.CharField(blank=True, default=None, max_length=500, null=True),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='icon_classes',
            field=models.CharField(blank=True, default=None, max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='text',
            field=models.CharField(blank=True, default=None, max_length=150, null=True),
        ),
    ]



# Generated by Django 1.10.2 on 2016-10-26 01:49

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0002_remove_eveapikeypair_error_count'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eveapikeypair',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='evecharacter',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 1.10.2 on 2016-12-12 03:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('mumble', '0002_auto_20161212_0100'),
    ]

    operations = [
        migrations.AddField(
            model_name='mumbleuser',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='mumble', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 4.0.10 on 2023-05-08 05:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0008_add_AA_GA-4_Team_Token '),
    ]

    operations = [
        migrations.RemoveField(
            model_name='analyticstokens',
            name='ignore_paths',
        ),
        migrations.RemoveField(
            model_name='analyticstokens',
            name='send_celery_tasks',
        ),
        migrations.RemoveField(
            model_name='analyticstokens',
            name='send_page_views',
        ),
        migrations.DeleteModel(
            name='AnalyticsPath',
        ),
    ]



from django.urls import path

from . import views

app_name = 'hrapplications'

urlpatterns = [
    path('', views.hr_application_management_view,
        name="index"),
    path('create/', views.hr_application_create_view,
        name="create_view"),
    path('create/<int:form_id>/', views.hr_application_create_view,
        name="create_view"),
    path('remove/<int:app_id>/', views.hr_application_remove,
        name="remove"),
    path('view/<int:app_id>/', views.hr_application_view,
        name="view"),
    path('personal/view/<int:app_id>/', views.hr_application_personal_view,
        name="personal_view"),
    path('personal/removal/<int:app_id>/',
        views.hr_application_personal_removal,
        name="personal_removal"),
    path('approve/<int:app_id>/', views.hr_application_approve,
        name="approve"),
    path('reject/<int:app_id>/', views.hr_application_reject,
        name="reject"),
    path('search/', views.hr_application_search,
        name="search"),
    path('mark_in_progress/<int:app_id>/', views.hr_application_mark_in_progress,
        name="mark_in_progress"),
]



from redis import Redis

from django.core.cache import caches

try:
    import django_redis
except ImportError:
    django_redis = None


def get_redis_client() -> Redis:
    """Get the configured redis client used by Django for caching.

    This function is a wrapper designed to work for both AA2 and AA3
    and should always be used to ensure backwards compatibility.
    """
    try:
        return django_redis.get_redis_connection("default")
    except AttributeError:
        default_cache = caches["default"]
        return default_cache.get_master_client()



# Generated by Django 1.10.2 on 2016-10-21 02:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0008_set_state'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='discourse_username',
        ),
        migrations.AddField(
            model_name='authservicesinfo',
            name='discourse_enabled',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 1.10.1 on 2016-09-05 21:40

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('level', models.CharField(choices=[(b'danger', b'CRITICAL'), (b'danger', b'ERROR'), (b'warning', b'WARN'), (b'info', b'INFO'), (b'success', b'DEBUG')], max_length=10)),
                ('title', models.CharField(max_length=254)),
                ('message', models.TextField()),
                ('timestamp', models.DateTimeField(auto_now_add=True)),
                ('viewed', models.BooleanField(default=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from unittest.mock import Mock, patch

from requests.exceptions import HTTPError

from django.contrib.auth.models import Group

from allianceauth.groupmanagement.models import ReservedGroupName
from allianceauth.tests.auth_utils import AuthUtils
from allianceauth.utils.testing import NoSocketsTestCase

from ..core import (
    _user_group_names,
    calculate_roles_for_user,
    group_to_role,
    server_name,
    user_formatted_nick,
)
from ..discord_client import DiscordApiBackoff, DiscordClient, RolesSet
from ..discord_client.tests.factories import TEST_USER_NAME, create_role
from . import MODULE_PATH, TEST_MAIN_ID, TEST_MAIN_NAME


class TestUserGroupNames(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.group_1 = Group.objects.create(name="Group 1")
        cls.group_2 = Group.objects.create(name="Group 2")

    def setUp(self):
        self.user = AuthUtils.create_member(TEST_USER_NAME)

    def test_return_groups_and_state_names_for_user(self):
        self.user.groups.add(self.group_1)
        result = _user_group_names(self.user)
        expected = ["Group 1", "Member"]
        self.assertSetEqual(set(result), set(expected))

    def test_return_state_only_if_user_has_no_groups(self):
        result = _user_group_names(self.user)
        expected = ["Member"]
        self.assertSetEqual(set(result), set(expected))


class TestUserFormattedNick(NoSocketsTestCase):
    def setUp(self):
        self.user = AuthUtils.create_user(TEST_USER_NAME)

    def test_return_nick_when_user_has_main(self):
        AuthUtils.add_main_character_2(self.user, TEST_MAIN_NAME, TEST_MAIN_ID)
        result = user_formatted_nick(self.user)
        expected = TEST_MAIN_NAME
        self.assertEqual(result, expected)

    def test_return_none_if_user_has_no_main(self):
        result = user_formatted_nick(self.user)
        self.assertIsNone(result)


@patch(MODULE_PATH + ".core.default_bot_client", spec=True)
class TestRoleForGroup(NoSocketsTestCase):
    def test_return_role_if_found(self, mock_bot_client):
        # given
        role = create_role(name="alpha")
        mock_bot_client.match_role_from_name.side_effect = (
            lambda guild_id, role_name: role if role.name == role_name else None
        )
        group = Group.objects.create(name="alpha")
        # when/then
        self.assertEqual(group_to_role(group), role)

    def test_return_empty_dict_if_not_found(self, mock_bot_client):
        # given
        role = create_role(name="alpha")
        mock_bot_client.match_role_from_name.side_effect = (
            lambda guild_id, role_name: role if role.name == role_name else None
        )
        group = Group.objects.create(name="unknown")
        # when/then
        self.assertIsNone(group_to_role(group))


@patch(MODULE_PATH + ".core.default_bot_client", spec=True)
@patch(MODULE_PATH + ".core.logger", spec=True)
class TestServerName(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.user = AuthUtils.create_user(TEST_USER_NAME)

    def test_returns_name_when_api_returns_it(self, mock_logger, mock_bot_client):
        # given
        my_server_name = "El Dorado"
        mock_bot_client.guild_name.return_value = my_server_name
        # when
        self.assertEqual(server_name(), my_server_name)
        # then
        self.assertFalse(mock_logger.warning.called)

    def test_returns_empty_string_when_api_throws_http_error(
        self, mock_logger, mock_bot_client
    ):
        mock_exception = HTTPError("Test exception")
        mock_exception.response = Mock(**{"status_code": 440})
        mock_bot_client.guild_name.side_effect = mock_exception

        self.assertEqual(server_name(), "")
        self.assertFalse(mock_logger.warning.called)

    def test_returns_empty_string_when_api_throws_service_error(
        self, mock_logger, mock_bot_client
    ):
        mock_bot_client.guild_name.side_effect = DiscordApiBackoff(1000)

        self.assertEqual(server_name(), "")
        self.assertFalse(mock_logger.warning.called)

    def test_returns_empty_string_when_api_throws_unexpected_error(
        self, mock_logger, mock_bot_client
    ):
        mock_bot_client.guild_name.side_effect = RuntimeError

        self.assertEqual(server_name(), "")
        self.assertTrue(mock_logger.warning.called)


class TestCalculateRolesForUser(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.user = AuthUtils.create_user(TEST_USER_NAME)

    def test_should_return_roles_for_new_member(self):
        # given
        roles = RolesSet([create_role()])
        my_client = Mock(spec=DiscordClient)
        my_client.guild_member_roles.return_value = RolesSet([])
        my_client.match_or_create_roles_from_names_2.return_value = roles
        # when
        roles_calculated, changed = calculate_roles_for_user(self.user, my_client, 42)
        # then
        self.assertTrue(changed)
        self.assertEqual(roles_calculated, roles)

    def test_should_return_changed_roles_for_existing_member(self):
        # given
        role_a = create_role()
        role_b = create_role()
        roles_current = RolesSet([role_a])
        roles_matching = RolesSet([role_a, role_b])
        my_client = Mock(spec=DiscordClient)
        my_client.guild_member_roles.return_value = roles_current
        my_client.match_or_create_roles_from_names_2.return_value = roles_matching
        # when
        roles_calculated, changed = calculate_roles_for_user(self.user, my_client, 42)
        # then
        self.assertTrue(changed)
        self.assertEqual(roles_calculated, roles_matching)

    def test_should_indicate_when_roles_are_unchanged(self):
        # given
        role_a = create_role()
        roles_current = RolesSet([role_a])
        roles_matching = RolesSet([role_a])
        my_client = Mock(spec=DiscordClient)
        my_client.guild_member_roles.return_value = roles_current
        my_client.match_or_create_roles_from_names_2.return_value = roles_matching
        # when
        roles_calculated, changed = calculate_roles_for_user(self.user, my_client, 42)
        # then
        self.assertFalse(changed)
        self.assertEqual(roles_calculated, roles_matching)

    def test_should_indicate_when_user_is_no_guild_member(self):
        # given
        role_a = create_role()
        roles_matching = RolesSet([role_a])
        my_client = Mock(spec=DiscordClient)
        my_client.guild_member_roles.return_value = None
        my_client.match_or_create_roles_from_names_2.return_value = roles_matching
        # when
        roles_calculated, changed = calculate_roles_for_user(self.user, my_client, 42)
        # then
        self.assertIsNone(changed)
        self.assertEqual(roles_calculated, roles_matching)

    def test_should_preserve_managed_roles_for_existing_member(self):
        # given
        role_a = create_role()
        role_b = create_role()
        role_m = create_role(managed=True)
        roles_current = RolesSet([role_a, role_m])
        roles_matching = RolesSet([role_b])
        my_client = Mock(spec=DiscordClient)
        my_client.guild_member_roles.return_value = roles_current
        my_client.match_or_create_roles_from_names_2.return_value = roles_matching
        # when
        roles_calculated, changed = calculate_roles_for_user(self.user, my_client, 42)
        # then
        self.assertTrue(changed)
        self.assertEqual(roles_calculated, RolesSet([role_b, role_m]))

    def test_should_preserve_reserved_roles_for_existing_member(self):
        # given
        role_a = create_role()
        role_b = create_role()
        role_c1 = create_role(name="charlie")
        role_c2 = create_role(name="Charlie")
        roles_current = RolesSet([role_a, role_c1, role_c2])
        roles_matching = RolesSet([role_b])
        my_client = Mock(spec=DiscordClient)
        my_client.guild_member_roles.return_value = roles_current
        my_client.match_or_create_roles_from_names_2.return_value = roles_matching
        ReservedGroupName.objects.create(
            name="charlie", reason="dummy", created_by="xyz"
        )
        # when
        roles_calculated, changed = calculate_roles_for_user(self.user, my_client, 42)
        # then
        self.assertTrue(changed)
        self.assertEqual(roles_calculated, RolesSet([role_b, role_c1, role_c2]))



import logging
from urllib.parse import urlencode

from requests.exceptions import HTTPError
from requests_oauthlib import OAuth2Session

from django.contrib.auth.models import Group, User
from django.db import models
from django.utils.timezone import now

from . import __title__
from .app_settings import (
    DISCORD_APP_ID,
    DISCORD_APP_SECRET,
    DISCORD_CALLBACK_URL,
    DISCORD_GUILD_ID,
    DISCORD_SYNC_NAMES,
)
from .core import calculate_roles_for_user, create_bot_client
from .core import group_to_role as core_group_to_role
from .core import server_name as core_server_name
from .core import user_formatted_nick
from .discord_client import (
    DISCORD_OAUTH_BASE_URL,
    DISCORD_OAUTH_TOKEN_URL,
    DiscordApiBackoff,
    DiscordClient,
)
from .utils import LoggerAddTag

logger = LoggerAddTag(logging.getLogger(__name__), __title__)


class DiscordUserManager(models.Manager):
    """Manager for DiscordUser"""

    # full server admin
    BOT_PERMISSIONS = 0x00000008

    # get user ID, accept invite
    SCOPES = [
        'identify',
        'guilds.join',
    ]

    def add_user(
        self,
        user: User,
        authorization_code: str,
        is_rate_limited: bool = True
    ) -> bool:
        """adds a new Discord user

        Params:
        - user: Auth user to join
        - authorization_code: authorization code returns from oauth
        - is_rate_limited: When False will disable default rate limiting (use with care)

        Returns: True on success, else False or raises exception
        """
        try:
            nickname = user_formatted_nick(user) if DISCORD_SYNC_NAMES else None
            access_token = self._exchange_auth_code_for_token(authorization_code)
            user_client = DiscordClient(access_token, is_rate_limited=is_rate_limited)
            discord_user = user_client.current_user()
            bot_client = create_bot_client(is_rate_limited=is_rate_limited)
            roles, changed = calculate_roles_for_user(
                user=user, client=bot_client, discord_uid=discord_user.id
            )
            if changed is None:
                # Handle new member
                created = bot_client.add_guild_member(
                    guild_id=DISCORD_GUILD_ID,
                    user_id=discord_user.id,
                    access_token=access_token,
                    role_ids=list(roles.ids()),
                    nick=nickname
                )
                if not created:
                    logger.warning(
                        "Failed to add user %s with Discord ID %s to Discord server",
                        user,
                        discord_user.id,
                    )
                    return False
            else:
                # Handle existing member
                logger.debug(
                    "User %s with Discord ID %s is already a member. Forcing a Refresh",
                    user,
                    discord_user.id,
                )
                # Force an update cause the discord API won't do it for us.
                updated = bot_client.modify_guild_member(
                    guild_id=DISCORD_GUILD_ID,
                    user_id=discord_user.id,
                    role_ids=list(roles.ids()),
                    nick=nickname
                )
                if not updated:
                    # Could not update the new user so fail.
                    logger.warning(
                        "Failed to add user %s with Discord ID %s to Discord server",
                        user,
                        discord_user.id,
                    )
                    return False

            self.update_or_create(
                user=user,
                defaults={
                    'uid': discord_user.id,
                    'username': discord_user.username[:32],
                    'discriminator': discord_user.discriminator[:4],
                    'activated': now()
                }
            )
            logger.info(
                "Added user %s with Discord ID %s to Discord server",
                user,
                discord_user.id
            )
            return True

        except (HTTPError, ConnectionError, DiscordApiBackoff) as ex:
            logger.exception(
                'Failed to add user %s to Discord server: %s', user, ex
            )
            return False

    def user_has_account(self, user: User) -> bool:
        """Returns True if the user has an Discord account, else False

        only checks locally, does not hit the API
        """
        if not isinstance(user, User):
            return False
        return self.filter(user=user).select_related('user').exists()

    @classmethod
    def generate_bot_add_url(cls) -> str:
        params = urlencode({
            'client_id': DISCORD_APP_ID,
            'scope': 'bot',
            'permissions': str(cls.BOT_PERMISSIONS)

        })
        return f'{DISCORD_OAUTH_BASE_URL}?{params}'

    @classmethod
    def generate_oauth_redirect_url(cls) -> str:
        oauth = OAuth2Session(
            DISCORD_APP_ID, redirect_uri=DISCORD_CALLBACK_URL, scope=cls.SCOPES
        )
        url, _ = oauth.authorization_url(DISCORD_OAUTH_BASE_URL)
        return url

    @staticmethod
    def _exchange_auth_code_for_token(authorization_code: str) -> str:
        oauth = OAuth2Session(DISCORD_APP_ID, redirect_uri=DISCORD_CALLBACK_URL)
        token = oauth.fetch_token(
            DISCORD_OAUTH_TOKEN_URL,
            client_secret=DISCORD_APP_SECRET,
            code=authorization_code
        )
        logger.debug("Received token from OAuth")
        return token['access_token']

    @staticmethod
    def group_to_role(group: Group) -> dict:
        """Fetch the Discord role matching the given Django group by name.

        Returns:
            - Discord role as dict
            - empty dict if no matching role found
        """
        role = core_group_to_role(group)
        return role.asdict() if role else dict()

    @staticmethod
    def server_name(use_cache: bool = True) -> str:
        """Fetches the name of the current Discord server.
        This method is kept to ensure backwards compatibility of this API.
        """
        return core_server_name(use_cache)



# Generated by Django 1.10.1 on 2016-12-14 21:48

from django.db import migrations

PERMISSIONS = {
    'user': [
        'corp_apis',
        'alliance_apis',
    ],
    'corpstats': {
        'corp_apis': 'Can view API keys of members of their corporation.',
        'alliance_apis': 'Can view API keys of members of their alliance.',
        'blue_apis': 'Can view API keys of members of blue corporations.',
        'view_corp_corpstats': 'Can view corp stats of their corporation.',
        'view_alliance_corpstats': 'Can view corp stats of members of their alliance.',
        'view_blue_corpstats': 'Can view corp stats of blue corporations.',
    }
}


def user_permissions_dict(apps):
    Permission = apps.get_model('auth', 'Permission')
    ContentType = apps.get_model('contenttypes', 'ContentType')
    User = apps.get_model('auth', 'User')
    CorpStats = apps.get_model('corputils', 'CorpStats')

    user_ct = ContentType.objects.get_for_model(User)
    corpstats_ct = ContentType.objects.get_for_model(CorpStats)

    return {
        'user': {x: Permission.objects.get_or_create(name=x, codename=x, content_type=user_ct)[0] for x in PERMISSIONS['user']},
        'corpstats': {x: Permission.objects.get_or_create(codename=x, name=y, content_type=corpstats_ct)[0] for x, y in PERMISSIONS['corpstats'].items()},
    }


def users_with_permission(apps, perm):
    User = apps.get_model('auth', 'User')
    return User.objects.filter(user_permissions=perm.pk)


def groups_with_permission(apps, perm):
    Group = apps.get_model('auth', 'Group')
    return Group.objects.filter(permissions=perm.pk)


def forward(apps, schema_editor):
    perm_dict = user_permissions_dict(apps)

    corp_users = users_with_permission(apps, perm_dict['user']['corp_apis'])
    for u in corp_users:
        u.user_permissions.add(perm_dict['corpstats']['corp_apis'].pk)
        u.user_permissions.add(perm_dict['corpstats']['view_corp_corpstats'].pk)

    alliance_users = users_with_permission(apps, perm_dict['user']['alliance_apis'])
    for u in alliance_users:
        u.user_permissions.add(perm_dict['corpstats']['alliance_apis'].pk)
        u.user_permissions.add(perm_dict['corpstats']['view_alliance_corpstats'].pk)

    corp_groups = groups_with_permission(apps, perm_dict['user']['corp_apis'])
    for g in corp_groups:
        g.permissions.add(perm_dict['corpstats']['corp_apis'].pk)
        g.permissions.add(perm_dict['corpstats']['view_corp_corpstats'].pk)

    alliance_groups = groups_with_permission(apps, perm_dict['user']['alliance_apis'])
    for g in alliance_groups:
        g.permissions.add(perm_dict['corpstats']['alliance_apis'].pk)
        g.permissions.add(perm_dict['corpstats']['view_alliance_corpstats'].pk)

    for name, perm in perm_dict['user'].items():
        perm.delete()


def reverse(apps, schema_editor):
    perm_dict = user_permissions_dict(apps)

    corp_users = users_with_permission(apps, perm_dict['corpstats']['view_corp_corpstats'])
    corp_api_users = users_with_permission(apps, perm_dict['corpstats']['corp_apis'])
    corp_us = corp_users | corp_api_users
    for u in corp_us.distinct():
        u.user_permissions.add(perm_dict['user']['corp_apis'].pk)
    for u in corp_users:
        u.user_permissions.remove(perm_dict['corpstats']['view_corp_corpstats'].pk)
    for u in corp_api_users:
        u.user_permissions.remove(perm_dict['corpstats']['corp_apis'].pk)

    alliance_users = users_with_permission(apps, perm_dict['corpstats']['view_alliance_corpstats'])
    alliance_api_users = users_with_permission(apps, perm_dict['corpstats']['alliance_apis'])
    alliance_us = alliance_users | alliance_api_users
    for u in alliance_us.distinct():
        u.user_permissions.add(perm_dict['user']['alliance_apis'].pk)
    for u in alliance_users:
        u.user_permissions.remove(perm_dict['corpstats']['view_alliance_corpstats'].pk)
    for u in alliance_api_users:
        u.user_permissions.remove(perm_dict['corpstats']['alliance_apis'].pk)

    corp_groups = groups_with_permission(apps, perm_dict['corpstats']['view_corp_corpstats'])
    corp_api_groups = groups_with_permission(apps, perm_dict['corpstats']['corp_apis'])
    for g in corp_groups.distinct():
        g.permissions.add(perm_dict['user']['corp_apis'].pk)
    for g in corp_groups:
        g.permissions.remove(perm_dict['corpstats']['view_corp_corpstats'].pk)
    for g in corp_api_groups:
        g.permissions.remove(perm_dict['corpstats']['corp_apis'].pk)

    alliance_groups = groups_with_permission(apps, perm_dict['corpstats']['view_alliance_corpstats'])
    alliance_api_groups = groups_with_permission(apps, perm_dict['corpstats']['alliance_apis'])
    alliance_gs = alliance_groups | alliance_api_groups
    for g in alliance_gs.distinct():
        g.permissions.add(perm_dict['user']['alliance_apis'].pk)
    for g in alliance_groups:
        g.permissions.remove(perm_dict['corpstats']['view_alliance_corpstats'].pk)
    for g in alliance_api_groups:
        g.permissions.remove(perm_dict['corpstats']['alliance_apis'].pk)


class Migration(migrations.Migration):

    dependencies = [
        ('corputils', '0001_initial'),
        ('authentication', '0005_delete_perms'),
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.RunPython(forward, reverse),
    ]



from unittest.mock import patch

from allianceauth.utils.testing import NoSocketsTestCase

from ..api import discord_guild_id
from . import MODULE_PATH


class TestDiscordGuildId(NoSocketsTestCase):
    @patch(MODULE_PATH + ".api.DISCORD_GUILD_ID", "123")
    def test_should_return_guild_id_when_configured(self):
        self.assertEqual(discord_guild_id(), 123)

    @patch(MODULE_PATH + ".api.DISCORD_GUILD_ID", "")
    def test_should_return_none_when_not_configured(self):
        self.assertIsNone(discord_guild_id())



from allianceauth.authentication import views
from django.urls import include, re_path, path

urlpatterns = [
    path('activate/complete/', views.activation_complete, name='registration_activation_complete'),
    # The activation key can make use of any character from the
    # URL-safe base64 alphabet, plus the colon as a separator.
    re_path(r'^activate/(?P<activation_key>[-:\w]+)/$', views.ActivationView.as_view(), name='registration_activate'),
    path('register/', views.RegistrationView.as_view(), name='registration_register'),
    path('register/complete/', views.registration_complete, name='registration_complete'),
    path('register/closed/', views.registration_closed, name='registration_disallowed'),
    path('', include('django.contrib.auth.urls')),
]



from unittest import TestCase
from unittest.mock import patch

from allianceauth.utils.counters import ItemCounter

MODULE_PATH = "allianceauth.utils.counters"

COUNTER_NAME = "test-counter"


class TestItemCounter(TestCase):
    def test_can_create_counter(self):
        # when
        counter = ItemCounter(COUNTER_NAME)
        # then
        self.assertIsInstance(counter, ItemCounter)

    def test_can_reset_counter_to_default(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        # when
        counter.reset()
        # then
        self.assertEqual(counter.value(), 0)

    def test_can_reset_counter_to_custom_value(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        # when
        counter.reset(42)
        # then
        self.assertEqual(counter.value(), 42)

    def test_can_increment_counter_by_default(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        counter.reset(0)
        # when
        counter.incr()
        # then
        self.assertEqual(counter.value(), 1)

    def test_can_increment_counter_by_custom_value(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        counter.reset(0)
        # when
        counter.incr(8)
        # then
        self.assertEqual(counter.value(), 8)

    def test_can_decrement_counter_by_default(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        counter.reset(9)
        # when
        counter.decr()
        # then
        self.assertEqual(counter.value(), 8)

    def test_can_decrement_counter_by_custom_value(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        counter.reset(9)
        # when
        counter.decr(8)
        # then
        self.assertEqual(counter.value(), 1)

    def test_can_decrement_counter_below_zero(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        counter.reset(0)
        # when
        counter.decr(1)
        # then
        self.assertEqual(counter.value(), -1)

    def test_can_not_decrement_counter_below_minimum(self):
        # given
        counter = ItemCounter(COUNTER_NAME, minimum=0)
        counter.reset(0)
        # when
        counter.decr(1)
        # then
        self.assertEqual(counter.value(), 0)

    def test_can_not_reset_counter_below_minimum(self):
        # given
        counter = ItemCounter(COUNTER_NAME, minimum=0)
        # when/then
        with self.assertRaises(ValueError):
            counter.reset(-1)

    def test_can_not_init_without_name(self):
        # when/then
        with self.assertRaises(ValueError):
            ItemCounter(name="")

    def test_can_ignore_invalid_values_when_incrementing(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        counter.reset(0)
        # when
        with patch(MODULE_PATH + ".cache.incr") as m:
            m.side_effect = ValueError
            counter.incr()
        # then
        self.assertEqual(counter.value(), 0)

    def test_can_ignore_invalid_values_when_decrementing(self):
        # given
        counter = ItemCounter(COUNTER_NAME)
        counter.reset(1)
        # when
        with patch(MODULE_PATH + ".cache.decr") as m:
            m.side_effect = ValueError
            counter.decr()
        # then
        self.assertEqual(counter.value(), 1)



import logging

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import user_passes_test
from django.contrib.auth.models import Group
from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
from django.db.models import Count
from django.http import Http404
from django.shortcuts import render, redirect, get_object_or_404
from django.utils.translation import gettext_lazy as _

from allianceauth.notifications import notify

from .managers import GroupManager
from .models import GroupRequest, RequestLog


logger = logging.getLogger(__name__)


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_management(request):
    logger.debug("group_management called by user %s" % request.user)
    acceptrequests = []
    leaverequests = []

    base_group_query = GroupRequest.objects.select_related('user', 'group', 'user__profile__main_character')
    if GroupManager.has_management_permission(request.user):
        # Full access
        group_requests = base_group_query.all()
    else:
        # Group specific leader
        users__groups = GroupManager.get_group_leaders_groups(request.user)
        group_requests = base_group_query.filter(group__in=users__groups)

    for grouprequest in group_requests:
        if grouprequest.leave_request:
            leaverequests.append(grouprequest)
        else:
            acceptrequests.append(grouprequest)

    logger.debug("Providing user {} with {} acceptrequests and {} leaverequests.".format(
        request.user, len(acceptrequests), len(leaverequests)))

    render_items = {
        'acceptrequests': acceptrequests,
        'leaverequests': leaverequests,
        'auto_leave': getattr(settings, 'GROUPMANAGEMENT_AUTO_LEAVE', False),
    }

    return render(request, 'groupmanagement/index.html', context=render_items)


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_membership(request):
    logger.debug("group_membership called by user %s" % request.user)
    # Get all open and closed groups
    if GroupManager.has_management_permission(request.user):
        # Full access
        groups = GroupManager.get_all_non_internal_groups()
    else:
        # Group leader specific
        groups = GroupManager.get_group_leaders_groups(request.user)

    groups = groups.exclude(authgroup__internal=True).annotate(num_members=Count('user')).order_by('name')

    render_items = {'groups': groups}

    return render(request, 'groupmanagement/groupmembership.html', context=render_items)


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_membership_audit(request, group_id):
    logger.debug("group_management_audit called by user %s" % request.user)
    group = get_object_or_404(Group, id=group_id)
    try:
        # Check its a joinable group i.e. not corp or internal
        # And the user has permission to manage it
        if not GroupManager.check_internal_group(group) or not GroupManager.can_manage_group(request.user, group):
            logger.warning(f"User {request.user} attempted to view the membership of group {group_id} but permission was denied")
            raise PermissionDenied

    except ObjectDoesNotExist:
        raise Http404("Group does not exist")
    render_items = {'group': group}
    entries = RequestLog.objects.filter(group=group).order_by('-date')
    render_items['entries'] = entries

    return render(request, 'groupmanagement/audit.html', context=render_items)


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_membership_list(request, group_id):
    logger.debug(
        "group_membership_list called by user %s "
        "for group id %s" % (request.user, group_id)
    )
    group = get_object_or_404(Group, id=group_id)
    try:

        # Check its a joinable group i.e. not corp or internal
        # And the user has permission to manage it
        if (not GroupManager.check_internal_group(group)
            or not GroupManager.can_manage_group(request.user, group)
        ):
            logger.warning(
                "User %s attempted to view the membership of group %s "
                "but permission was denied" % (request.user, group_id)
            )
            raise PermissionDenied

    except ObjectDoesNotExist:
        raise Http404("Group does not exist")

    group_leaders = group.authgroup.group_leaders.all()
    members = list()
    for member in \
        group.user_set\
            .all()\
            .select_related('profile', 'profile__main_character')\
            .order_by('profile__main_character__character_name'):

        members.append({
            'user': member,
            'main_char': member.profile.main_character,
            'is_leader': member in group_leaders
        })

    render_items = {'group': group, 'members': members}

    return render(
        request, 'groupmanagement/groupmembers.html',
        context=render_items
    )


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_membership_remove(request, group_id, user_id):
    logger.debug(f"group_membership_remove called by user {request.user} for group id {group_id} on user id {user_id}")
    group = get_object_or_404(Group, id=group_id)
    try:
        # Check its a joinable group i.e. not corp or internal
        # And the user has permission to manage it
        if not GroupManager.check_internal_group(group) or not GroupManager.can_manage_group(request.user, group):
            logger.warning(f"User {request.user} attempted to remove a user from group {group_id} but permission was denied")
            raise PermissionDenied

        try:
            user = group.user_set.get(id=user_id)
            request_info = user.username + ":" + group.name
            log = RequestLog(request_type=None,group=group,request_info=request_info,action=1,request_actor=request.user)
            log.save()
            # Remove group from user
            user.groups.remove(group)
            logger.info(f"User {request.user} removed user {user} from group {group}")
            messages.success(request, _("Removed user %(user)s from group %(group)s.") % {"user": user, "group": group})
        except ObjectDoesNotExist:
            messages.warning(request, _("User does not exist in that group"))

    except ObjectDoesNotExist:
        messages.warning(request, _("Group does not exist"))

    return redirect('groupmanagement:membership', group_id)


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_accept_request(request, group_request_id):
    logger.debug(f"group_accept_request called by user {request.user} for grouprequest id {group_request_id}")
    group_request = get_object_or_404(GroupRequest, id=group_request_id)
    try:
        group, created = Group.objects.get_or_create(name=group_request.group.name)

        if not GroupManager.joinable_group(group_request.group, group_request.user.profile.state) or \
                not GroupManager.can_manage_group(request.user, group_request.group):
            raise PermissionDenied

        group_request.user.groups.add(group)
        group_request.user.save()
        log = RequestLog(request_type=group_request.leave_request,group=group,request_info=group_request.__str__(),action=1,request_actor=request.user)
        log.save()
        group_request.delete()
        logger.info("User {} accepted group request from user {} to group {}".format(
            request.user, group_request.user, group_request.group.name))
        notify(group_request.user, "Group Application Accepted", level="success",
                message="Your application to %s has been accepted." % group_request.group)
        messages.success(request,
                        _('Accepted application from %(mainchar)s to %(group)s.') % {"mainchar": group_request.main_char, "group": group_request.group})

    except PermissionDenied as p:
        logger.warning(f"User {request.user} attempted to accept group join request {group_request_id} but permission was denied")
        raise p
    except:
        messages.error(request, _('An unhandled error occurred while processing the application from %(mainchar)s to %(group)s.') % {"mainchar": group_request.main_char, "group": group_request.group})
        logger.exception("Unhandled exception occurred while user {} attempting to accept grouprequest id {}.".format(
            request.user, group_request_id))
        pass

    return redirect("groupmanagement:management")


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_reject_request(request, group_request_id):
    logger.debug(f"group_reject_request called by user {request.user} for group request id {group_request_id}")
    group_request = get_object_or_404(GroupRequest, id=group_request_id)
    try:
        if not GroupManager.can_manage_group(request.user, group_request.group):
            raise PermissionDenied

        if group_request:
            logger.info("User {} rejected group request from user {} to group {}".format(
                request.user, group_request.user, group_request.group.name))
            log = RequestLog(request_type=group_request.leave_request,group=group_request.group,request_info=group_request.__str__(),action=0,request_actor=request.user)
            log.save()
            group_request.delete()
            notify(group_request.user, "Group Application Rejected", level="danger", message="Your application to %s has been rejected." % group_request.group)
            messages.success(request,
                            _('Rejected application from %(mainchar)s to %(group)s.') % {"mainchar": group_request.main_char, "group": group_request.group})

    except PermissionDenied as p:
        logger.warning(f"User {request.user} attempted to reject group join request {group_request_id} but permission was denied")
        raise p
    except:
        messages.error(request, _('An unhandled error occurred while processing the application from %(mainchar)s to %(group)s.') % {"mainchar": group_request.main_char, "group": group_request.group})
        logger.exception("Unhandled exception occurred while user {} attempting to reject group request id {}".format(
            request.user, group_request_id))
        pass

    return redirect("groupmanagement:management")


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_leave_accept_request(request, group_request_id):
    logger.debug(
        f"group_leave_accept_request called by user {request.user} for group request id {group_request_id}")
    group_request = get_object_or_404(GroupRequest, id=group_request_id)
    try:
        if not GroupManager.can_manage_group(request.user, group_request.group):
            raise PermissionDenied

        group, created = Group.objects.get_or_create(name=group_request.group.name)
        group_request.user.groups.remove(group)
        group_request.user.save()
        log = RequestLog(request_type=group_request.leave_request,group=group_request.group,request_info=group_request.__str__(),action=1,request_actor=request.user)
        log.save()
        group_request.delete()
        logger.info("User {} accepted group leave request from user {} to group {}".format(
            request.user, group_request.user, group_request.group.name))
        notify(group_request.user, "Group Leave Request Accepted", level="success",
                message="Your request to leave %s has been accepted." % group_request.group)
        messages.success(request,
                        _('Accepted application from %(mainchar)s to leave %(group)s.') % {"mainchar": group_request.main_char, "group": group_request.group})
    except PermissionDenied as p:
        logger.warning(f"User {request.user} attempted to accept group leave request {group_request_id} but permission was denied")
        raise p
    except:
        messages.error(request, _('An unhandled error occurred while processing the application from %(mainchar)s to leave %(group)s.') % {
            "mainchar": group_request.main_char, "group": group_request.group})
        logger.exception("Unhandled exception occurred while user {} attempting to accept group leave request id {}".format(
            request.user, group_request_id))
        pass

    return redirect("groupmanagement:management")


@login_required
@user_passes_test(GroupManager.can_manage_groups)
def group_leave_reject_request(request, group_request_id):
    logger.debug(
        f"group_leave_reject_request called by user {request.user} for group request id {group_request_id}")
    group_request = get_object_or_404(GroupRequest, id=group_request_id)
    try:
        if not GroupManager.can_manage_group(request.user, group_request.group):
            raise PermissionDenied

        if group_request:
            log = RequestLog(request_type=group_request.leave_request,group=group_request.group,request_info=group_request.__str__(),action=0,request_actor=request.user)
            log.save()
            group_request.delete()
            logger.info("User {} rejected group leave request from user {} for group {}".format(
                request.user, group_request.user, group_request.group.name))
            notify(group_request.user, "Group Leave Request Rejected", level="danger", message="Your request to leave %s has been rejected." % group_request.group)
            messages.success(request, _('Rejected application from %(mainchar)s to leave %(group)s.') % {
                "mainchar": group_request.main_char, "group": group_request.group})
    except PermissionDenied as p:
        logger.warning(f"User {request.user} attempted to reject group leave request {group_request_id} but permission was denied")
        raise p
    except:
        messages.error(request, _('An unhandled error occurred while processing the application from %(mainchar)s to leave %(group)s.') % {
            "mainchar": group_request.main_char, "group": group_request.group})
        logger.exception("Unhandled exception occurred while user {} attempting to reject group leave request id {}".format(
            request.user, group_request_id))
        pass

    return redirect("groupmanagement:management")


@login_required
def groups_view(request):
    logger.debug("groups_view called by user %s" % request.user)

    groups_qs = GroupManager.get_joinable_groups_for_user(
        request.user, include_hidden=False
    )
    groups_qs = groups_qs.order_by('name').select_related("authgroup").prefetch_related('authgroup__group_leaders', 'authgroup__group_leaders__profile__main_character', 'authgroup__group_leader_groups')
    groups = []

    ## TODO see about making this faster
    for group in groups_qs:
        group_request = GroupRequest.objects\
            .filter(user=request.user)\
            .filter(group=group)
        groups.append({
            'group': group,
            'request': group_request[0] if group_request else None
        })

    count = 0
    perms = GroupManager.can_manage_groups(request.user)
    if perms:
        count = GroupManager.pending_requests_count_for_user(request.user)

    user_groups_list = list(request.user.groups.all())
    context = {'groups': groups, "manager_perms": perms, "req_count":count, "user_groups": user_groups_list}

    return render(request, 'groupmanagement/groups.html', context=context)


@login_required
def group_request_add(request, group_id):
    logger.debug(f"group_request_add called by user {request.user} for group id {group_id}")
    group = Group.objects.get(id=group_id)
    state = request.user.profile.state
    if not GroupManager.joinable_group(group, state):
        logger.warning(f"User {request.user} attempted to join group id {group_id} but it is not a joinable group")
        messages.warning(request, _("You cannot join that group"))
        return redirect('groupmanagement:groups')
    if group in request.user.groups.all():
        # User is already a member of this group.
        logger.warning(f"User {request.user} attempted to join group id {group_id} but they are already a member.")
        messages.warning(request, _("You are already a member of that group."))
        return redirect('groupmanagement:groups')
    if not request.user.has_perm('groupmanagement.request_groups') and not group.authgroup.public:
        # Does not have the required permission, trying to join a non-public group
        logger.warning(f"User {request.user} attempted to join group id {group_id} but it is not a public group")
        messages.warning(request, _("You cannot join that group"))
        return redirect('groupmanagement:groups')
    if group.authgroup.open:
        logger.info(f"{request.user} joining {group} as is an open group")
        request.user.groups.add(group)
        request_info = request.user.username + ":" + group.name
        log = RequestLog(request_type=False, group=group, request_info=request_info, action=1, request_actor=request.user)
        log.save()
        return redirect("groupmanagement:groups")
    req = GroupRequest.objects.filter(user=request.user, group=group)
    if len(req) > 0:
        logger.info(f"{request.user} attempted to join {group} but already has an open application")
        messages.warning(request, _("You already have a pending application for that group."))
        return redirect("groupmanagement:groups")
    grouprequest = GroupRequest()
    grouprequest.group = group
    grouprequest.user = request.user
    grouprequest.leave_request = False
    grouprequest.save()
    logger.info(f"Created group request for user {request.user} to group {Group.objects.get(id=group_id)}")
    grouprequest.notify_leaders()
    messages.success(request, _('Applied to group %(group)s.') % {"group": group})
    return redirect("groupmanagement:groups")


@login_required
def group_request_leave(request, group_id):
    logger.debug(f"group_request_leave called by user {request.user} for group id {group_id}")
    group = Group.objects.get(id=group_id)
    if not GroupManager.check_internal_group(group):
        logger.warning(f"User {request.user} attempted to leave group id {group_id} but it is not a joinable group")
        messages.warning(request, _("You cannot leave that group"))
        return redirect('groupmanagement:groups')
    if group not in request.user.groups.all():
        logger.debug(f"User {request.user} attempted to leave group id {group_id} but they are not a member")
        messages.warning(request, _("You are not a member of that group"))
        return redirect('groupmanagement:groups')
    if group.authgroup.open:
        logger.info(f"{request.user} leaving {group} as is an open group")
        request_info = request.user.username + ":" + group.name
        log = RequestLog(request_type=True, group=group, request_info=request_info, action=1, request_actor=request.user)
        log.save()
        request.user.groups.remove(group)
        return redirect("groupmanagement:groups")
    req = GroupRequest.objects.filter(user=request.user, group=group)
    if len(req) > 0:
        logger.info(f"{request.user} attempted to leave {group} but already has an pending leave request.")
        messages.warning(request, _("You already have a pending leave request for that group."))
        return redirect("groupmanagement:groups")
    if getattr(settings, 'GROUPMANAGEMENT_AUTO_LEAVE', False):
        logger.info(f"{request.user} leaving joinable group {group} due to auto_leave")
        request_info = request.user.username + ":" + group.name
        log = RequestLog(request_type=True, group=group, request_info=request_info, action=1, request_actor=request.user)
        log.save()
        request.user.groups.remove(group)
        return redirect('groupmanagement:groups')
    grouprequest = GroupRequest()
    grouprequest.group = group
    grouprequest.user = request.user
    grouprequest.leave_request = True
    grouprequest.save()
    logger.info(f"Created group leave request for user {request.user} to group {Group.objects.get(id=group_id)}")
    grouprequest.notify_leaders()
    messages.success(request, _('Applied to leave group %(group)s.') % {"group": group})
    return redirect("groupmanagement:groups")



# flake8: noqa

"""Concurrency testing Discord service tasks

This script will run many Discord service tasks in parallel to test concurrency
Note that it will run against your main Auth database and not test!

Check allianceauth.log for the results.

To run this test start a bunch of celery workers and then run this script directly.
Make sure to also set the environment variable AUTH_PROJECT_PATH to your Auth path
and DJANGO_SETTINGS_MODULE to the relative location of your settings:

Example:
export AUTH_PROJECT_PATH="/home/erik997/dev/python/aa/allianceauth-dev/myauth"
export DJANGO_SETTINGS_MODULE="myauth.settings.local"

Careful: This script will utilize all existing Discord users and make changes!
"""
# start django project
import os
import sys
if not 'AUTH_PROJECT_PATH' in os.environ:
    print('AUTH_PROJECT_PATH is not set')
    exit(1)
if not 'DJANGO_SETTINGS_MODULE' in os.environ:
    print('DJANGO_SETTINGS_MODULE is not set')
    exit(1)
sys.path.insert(0, os.environ['AUTH_PROJECT_PATH'])
import django
django.setup()

# normal imports
import logging
from uuid import uuid1
import random

from django.contrib.auth.models import User, Group

from allianceauth.services.modules.discord.models import DiscordUser
from allianceauth.utils.cache import get_redis_client

logger = logging.getLogger('allianceauth')
MAX_RUNS = 3


def clear_cache():
    redis = get_redis_client()
    redis.flushall()
    logger.info('Cache flushed')

def run_many_updates(runs):
    logger.info('Starting piloting_tasks for %d runs', runs)
    users = list()
    all_groups = Group.objects.all()
    for i in range(runs):
        if not users:
            users = list(User.objects.filter(discord__isnull=False))
        user = users.pop()
        logger.info('%d/%d: Starting run with user %s', i + 1, runs, user)
        # force change of nick
        new_nick = f'Testnick {uuid1().hex}'[:32]
        logger.info(
            '%d/%d: Changing nickname of %s to "%s"', i + 1, runs, user, new_nick
        )
        user.profile.main_character.character_name = new_nick
        user.profile.main_character.save()

        # force change of groups
        user_groups = user.groups.all()
        user.groups.remove(random.choice(user_groups))
        while True:
            new_group = random.choice(all_groups)
            if new_group not in user_groups:
                break
        logger.info('%d/%d: Adding group "%s" to user %s', i + 1, runs, new_group, user)
        user.groups.add(new_group)

    logger.info('All %d runs have been started', runs)

if __name__ == "__main__":
    clear_cache()
    run_many_updates(MAX_RUNS)



import logging

from django.contrib.auth.models import Group, User
from django.db.models import Q, QuerySet

from allianceauth.authentication.models import State
from .models import GroupRequest


logger = logging.getLogger(__name__)


class GroupManager:

    @classmethod
    def get_joinable_groups_for_user(
        cls, user: User, include_hidden=True
    ) -> QuerySet[Group]:
        """get groups a user could join incl. groups already joined"""
        groups_qs = cls.get_joinable_groups(user.profile.state)

        if not user.has_perm('groupmanagement.request_groups'):
            groups_qs = groups_qs.filter(authgroup__public=True)

        if not include_hidden:
            groups_qs = groups_qs.filter(authgroup__hidden=False)

        return groups_qs

    @staticmethod
    def get_joinable_groups(state: State) -> QuerySet[Group]:
        """get groups that can be joined by user with given state"""
        return (
            Group.objects
            .exclude(authgroup__internal=True)
            .filter(Q(authgroup__states=state) | Q(authgroup__states=None))
        )

    @staticmethod
    def get_all_non_internal_groups() -> QuerySet[Group]:
        """get groups that are not internal"""
        return Group.objects.exclude(authgroup__internal=True)

    @staticmethod
    def get_group_leaders_groups(user: User) -> QuerySet[Group]:
        return (
            Group.objects.filter(authgroup__group_leaders=user)
            | Group.objects.filter(
                authgroup__group_leader_groups__in=list(user.groups.all())
            )
        )

    @staticmethod
    def joinable_group(group: Group, state: State) -> bool:
        """
        Check if a group is a user/state joinable group, i.e.
        not an internal group for Corp, Alliance, Members etc,
        or restricted from the user's current state.
        :param group: django.contrib.auth.models.Group object
        :param state: allianceauth.authentication.State object
        :return: bool True if its joinable, False otherwise
        """
        if (
            len(group.authgroup.states.all()) != 0
            and state not in group.authgroup.states.all()
        ):
            return False
        return not group.authgroup.internal

    @staticmethod
    def check_internal_group(group: Group) -> bool:
        """
        Check if a group is auditable, i.e not an internal group
        :param group: django.contrib.auth.models.Group object
        :return: bool True if it is auditable, false otherwise
        """
        return not group.authgroup.internal

    @staticmethod
    def has_management_permission(user: User) -> bool:
        return user.has_perm('auth.group_management')

    @classmethod
    def can_manage_groups(cls, user:User) -> bool:
        """
        For use with user_passes_test decorator.
        Check if the user can manage groups. Either has the
        auth.group_management permission or is a leader of at least one group
        and is also a Member.
        :param user: django.contrib.auth.models.User for the request
        :return: bool True if user can manage groups, False otherwise
        """
        if user.is_authenticated:
            return (
                cls.has_management_permission(user)
                or cls.get_group_leaders_groups(user)
            )
        return False

    @classmethod
    def can_manage_group(cls, user: User, group: Group) -> bool:
        """
        Check user has permission to manage the given group
        :param user: User object to test permission of
        :param group: Group object the user is attempting to manage
        :return: True if the user can manage the group
        """
        if user.is_authenticated:
            return (
                cls.has_management_permission(user)
                or cls.get_group_leaders_groups(user).filter(pk=group.pk).exists()
            )
        return False

    @classmethod
    def pending_requests_count_for_user(cls, user: User) -> int:
        """Returns the number of pending group requests for the given user"""
        if cls.has_management_permission(user):
            return GroupRequest.objects.all().count()
        return (
            GroupRequest.objects
            .filter(group__in=list(cls.get_group_leaders_groups(user)))
            .count()
        )



from django.test import TestCase
from django.contrib.auth.models import User, Group

from allianceauth.eveonline.models import EveCorporationInfo
from allianceauth.eveonline.autogroups.models import AutogroupsConfig
from allianceauth.tests.auth_utils import AuthUtils


from ..models import ReservedGroupName


class TestGroupSignals(TestCase):
    def test_should_create_authgroup_when_group_is_created(self):
        # when
        group = Group.objects.create(name="test")
        # then
        self.assertEqual(group.authgroup.group, group)

    def test_should_rename_group_that_conflicts_with_reserved_name(self):
        # given
        ReservedGroupName.objects.create(name="test", reason="dummy", created_by="xyz")
        ReservedGroupName.objects.create(name="test_1", reason="dummy", created_by="xyz")
        # when
        group = Group.objects.create(name="Test")
        # then
        self.assertNotEqual(group.name, "test")
        self.assertNotEqual(group.name, "test_1")


class TestCheckGroupsOnStateChange(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test')
        cls.character = AuthUtils.add_main_character_2(
            cls.user, 'test character', 1001, corp_id=2001, corp_name='test corp 1', corp_ticker='TEST'
        )
        cls.user.profile.refresh_from_db()
        cls.corp_1 = EveCorporationInfo.objects.create(
            corporation_id=2001, corporation_name='test corp 1', corporation_ticker='C1', member_count=1
        )
        cls.corp_2 = EveCorporationInfo.objects.create(
            corporation_id=2002, corporation_name='test corp 2', corporation_ticker='C2', member_count=1
        )
        cls.guest_state = AuthUtils.get_guest_state()
        cls.test_state_1 = AuthUtils.create_state('test_state_1', 500)
        cls.test_state_2 = AuthUtils.create_state('test_state_2', 600)

    def setUp(self):
        self.user.refresh_from_db()

    def _refresh_user(self):
        self.user = User.objects.get(pk=self.user.pk)

    def test_drop_state_group(self):
        """
        given user is member of: state group, normal group and auto group
        when user looses state
        then user is automatically kicked from state group
        and remains member of normal group and auto group
        """
        # setup
        state_group = Group.objects.create(name='state_group')
        state_group.authgroup.states.add(self.test_state_1)
        state_group.authgroup.internal = False
        state_group.save()
        normal_group = Group.objects.create(name='normal_group')
        normal_group.authgroup.internal = False
        normal_group.save()
        internal_group = Group.objects.create(name='internal_group')
        autogroup_config = AutogroupsConfig.objects.create(corp_groups=True)
        autogroup_config.states.add(self.test_state_1)
        autogroup_config.states.add(self.guest_state)
        auto_group = autogroup_config.corp_managed_groups.first()
        internal_state_group = Group.objects.create(name='internal_state_group')
        internal_state_group.authgroup.states.add(self.test_state_1)
        self.test_state_1.member_corporations.add(self.corp_1)
        self.user.groups.add(normal_group)
        self.user.groups.add(internal_group)
        self.user.groups.add(state_group)
        self.user.groups.add(internal_state_group)

        # user changes state back to guest
        self.test_state_1.member_corporations.clear()

        # assert
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.guest_state)
        groups = self.user.groups.all()
        self.assertNotIn(state_group, groups)  # looses state group
        self.assertNotIn(internal_state_group, groups)  # looses state group
        self.assertIn(normal_group, groups)  # normal group unafected
        self.assertIn(internal_group, groups)  # internal group unafected
        self.assertIn(auto_group, groups)  # auto group unafected

    def test_change_to_other_state(self):
        """
        given a state group with 2 allowed states
        when user changes from one state to the other
        then user remains member of that group
        """
        # setup
        state_group = Group.objects.create(name='state_group')
        state_group.authgroup.states.add(self.test_state_1)
        state_group.authgroup.states.add(self.test_state_2)

        self.test_state_1.member_corporations.add(self.corp_1)
        self.test_state_2.member_corporations.add(self.corp_2)
        self.user.groups.add(state_group)

        # user changes state back to guest
        self.character.corporation_id = 2002
        self.character.corporation_name = "test corp 2"
        self.character.save()

        # assert
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.test_state_2)
        groups = self.user.groups.all()
        self.assertIn(state_group, groups)



"""Core functionality of the Discord service not directly related to models."""

import logging
from typing import List, Optional, Tuple

from requests.exceptions import HTTPError

from django.contrib.auth.models import Group, User

from allianceauth.groupmanagement.models import ReservedGroupName
from allianceauth.services.hooks import NameFormatter

from . import __title__
from .app_settings import DISCORD_BOT_TOKEN, DISCORD_GUILD_ID
from .discord_client import DiscordClient, RolesSet, Role
from .discord_client.exceptions import DiscordClientException
from .utils import LoggerAddTag

logger = LoggerAddTag(logging.getLogger(__name__), __title__)


def create_bot_client(is_rate_limited: bool = True) -> DiscordClient:
    """Create new bot client for accessing the configured Discord server.

    Args:
        is_rate_limited: Set to False to turn off rate limiting (use with care).

    Return:
        Discord client instance
    """
    return DiscordClient(DISCORD_BOT_TOKEN, is_rate_limited=is_rate_limited)


def calculate_roles_for_user(
    user: User,
    client: DiscordClient,
    discord_uid: int,
    state_name: str = None,
) -> Tuple[RolesSet, Optional[bool]]:
    """Calculate current Discord roles for an Auth user.

    Takes into account reserved groups and existing managed roles (e.g. nitro).

    Returns:
        - Discord roles, changed flag:
            - True when roles have changed,
            - False when they have not changed,
            - None if user is not a member of the guild
    """
    roles_calculated = client.match_or_create_roles_from_names_2(
        guild_id=DISCORD_GUILD_ID,
        role_names=_user_group_names(user=user, state_name=state_name),
    )
    logger.debug("Calculated roles for user %s: %s", user, roles_calculated.ids())
    roles_current = client.guild_member_roles(
        guild_id=DISCORD_GUILD_ID, user_id=discord_uid
    )
    if roles_current is None:
        logger.debug("User %s is not a member of the guild.", user)
        return roles_calculated, None
    logger.debug("Current roles user %s: %s", user, roles_current.ids())
    reserved_role_names = ReservedGroupName.objects.values_list("name", flat=True)
    roles_reserved = roles_current.subset(role_names=reserved_role_names)
    roles_managed = roles_current.subset(managed_only=True)
    roles_persistent = roles_managed.union(roles_reserved)
    if roles_calculated == roles_current.difference(roles_persistent):
        return roles_calculated, False
    return roles_calculated.union(roles_persistent), True


def _user_group_names(user: User, state_name: str = None) -> List[str]:
    """Names of groups and state the given user is a member of."""
    if not state_name:
        state_name = user.profile.state.name
    group_names = [group.name for group in user.groups.all()] + [state_name]
    logger.debug("Group names for roles updates of user %s are: %s", user, group_names)
    return group_names


def user_formatted_nick(user: User) -> Optional[str]:
    """Name of the given user's main character with name formatting applied.

    Returns:
        Name or ``None`` if user has no main.
    """
    from .auth_hooks import DiscordService

    if user.profile.main_character:
        return NameFormatter(DiscordService(), user).format_name()
    return None


def group_to_role(group: Group) -> Optional[Role]:
    """Fetch the Discord role matching the given Django group by name.

    Returns:
        Discord role or None if no matching role exist
    """
    return default_bot_client.match_role_from_name(
        guild_id=DISCORD_GUILD_ID, role_name=group.name
    )


def server_name(use_cache: bool = True) -> str:
    """Fetches the name of the current Discord server.

    Args:
        use_cache: When set False will force an API call to get the server name

    Returns:
        Server name or an empty string if the name could not be retrieved
    """
    try:
        server_name = default_bot_client.guild_name(
            guild_id=DISCORD_GUILD_ID, use_cache=use_cache
        )
    except (HTTPError, DiscordClientException):
        server_name = ""
    except Exception:
        logger.warning(
            "Unexpected error when trying to retrieve the server name from Discord",
            exc_info=True,
        )
        server_name = ""
    return server_name


# Default bot client to be used by modules of this package
default_bot_client = create_bot_client()



from django import forms
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from .providers import ObjectNotFound

from .models import EveAllianceInfo
from .models import EveCharacter
from .models import EveCorporationInfo
from .models import EveFactionInfo


class EveEntityExistsError(forms.ValidationError):
    def __init__(self, entity_type_name, id):
        super().__init__(
            message=f'{entity_type_name} with ID {id} already exists.')


class EveEntityNotFoundError(forms.ValidationError):
    def __init__(self, entity_type_name, id):
        super().__init__(
            message=f'{entity_type_name} with ID {id} not found.')


class EveEntityForm(forms.ModelForm):
    id = forms.IntegerField(min_value=1)
    entity_model_class = None

    def clean_id(self):
        raise NotImplementedError()

    def save(self, commit=True):
        raise NotImplementedError()

    def save_m2m(self):
        pass


def get_faction_choices():
    # use a method to avoid making an ESI call when the app loads
    # restrict to only those factions a player can join for faction warfare
    player_factions = [x for x in EveFactionInfo.provider.get_all_factions() if x['militia_corporation_id']]
    return [(x['faction_id'], x['name']) for x in player_factions]



class EveFactionForm(EveEntityForm):
    id = forms.ChoiceField(
        choices=get_faction_choices,
        label="Name"
    )

    def clean_id(self):
        try:
            assert self.Meta.model.provider.get_faction(self.cleaned_data['id'])
        except (AssertionError, ObjectNotFound):
            raise EveEntityNotFoundError('faction', self.cleaned_data['id'])
        if self.Meta.model.objects.filter(faction_id=self.cleaned_data['id']).exists():
            raise EveEntityExistsError('faction', self.cleaned_data['id'])
        return self.cleaned_data['id']

    def save(self, commit=True):
        faction = self.Meta.model.provider.get_faction(self.cleaned_data['id'])
        return self.Meta.model.objects.create(faction_id=faction.id, faction_name=faction.name)

    class Meta:
        fields = ['id']
        model = EveFactionInfo


class EveCharacterForm(EveEntityForm):
    entity_type_name = 'character'

    def clean_id(self):
        try:
            assert self.Meta.model.provider.get_character(self.cleaned_data['id'])
        except (AssertionError, ObjectNotFound):
            raise EveEntityNotFoundError(self.entity_type_name, self.cleaned_data['id'])
        if self.Meta.model.objects.filter(character_id=self.cleaned_data['id']).exists():
            raise EveEntityExistsError(self.entity_type_name, self.cleaned_data['id'])
        return self.cleaned_data['id']

    def save(self, commit=True):
        return self.Meta.model.objects.create_character(self.cleaned_data['id'])

    class Meta:
        fields = ['id']
        model = EveCharacter


class EveCorporationForm(EveEntityForm):
    entity_type_name = 'corporation'

    def clean_id(self):
        try:
            assert self.Meta.model.provider.get_corporation(self.cleaned_data['id'])
        except (AssertionError, ObjectNotFound):
            raise EveEntityNotFoundError(self.entity_type_name, self.cleaned_data['id'])
        if self.Meta.model.objects.filter(corporation_id=self.cleaned_data['id']).exists():
            raise EveEntityExistsError(self.entity_type_name, self.cleaned_data['id'])
        return self.cleaned_data['id']

    def save(self, commit=True):
        return self.Meta.model.objects.create_corporation(self.cleaned_data['id'])

    class Meta:
        fields = ['id']
        model = EveCorporationInfo


class EveAllianceForm(EveEntityForm):
    entity_type_name = 'alliance'

    def clean_id(self):
        try:
            assert self.Meta.model.provider.get_alliance(self.cleaned_data['id'])
        except (AssertionError, ObjectNotFound):
            raise EveEntityNotFoundError(self.entity_type_name, self.cleaned_data['id'])
        if self.Meta.model.objects.filter(alliance_id=self.cleaned_data['id']).exists():
            raise EveEntityExistsError(self.entity_type_name, self.cleaned_data['id'])
        return self.cleaned_data['id']

    def save(self, commit=True):
        return self.Meta.model.objects.create_alliance(self.cleaned_data['id'])

    class Meta:
        fields = ['id']
        model = EveAllianceInfo


@admin.register(EveFactionInfo)
class EveFactionInfoAdmin(admin.ModelAdmin):
    search_fields = ['faction_name']
    list_display = ('faction_name',)
    ordering = ('faction_name',)

    def has_change_permission(self, request, obj=None):
        return False

    def get_form(self, request, obj=None, **kwargs):
        if not obj or not obj.pk:
            return EveFactionForm
        return super().get_form(request, obj=obj, **kwargs)


@admin.register(EveCorporationInfo)
class EveCorporationInfoAdmin(admin.ModelAdmin):
    search_fields = ['corporation_name']
    list_display = ('corporation_name', 'alliance')
    list_select_related = ('alliance',)
    list_filter = (('alliance', admin.RelatedOnlyFieldListFilter),)
    ordering = ('corporation_name',)

    def has_change_permission(self, request, obj=None):
        return False

    def get_form(self, request, obj=None, **kwargs):
        if not obj or not obj.pk:
            return EveCorporationForm
        return super().get_form(request, obj=obj, **kwargs)


@admin.register(EveAllianceInfo)
class EveAllianceInfoAdmin(admin.ModelAdmin):
    search_fields = ['alliance_name']
    list_display = ('alliance_name',)
    ordering = ('alliance_name',)

    def has_change_permission(self, request, obj=None):
        return False

    def get_form(self, request, obj=None, **kwargs):
        if not obj or not obj.pk:
            return EveAllianceForm
        return super().get_form(request, obj=obj, **kwargs)


@admin.register(EveCharacter)
class EveCharacterAdmin(admin.ModelAdmin):
    search_fields = [
        'character_name',
        'corporation_name',
        'alliance_name',
        'character_ownership__user__username'
    ]
    list_display = (
        'character_name', 'corporation_name', 'alliance_name', 'faction_name', 'user', 'main_character'
    )
    list_select_related = (
        'character_ownership', 'character_ownership__user__profile__main_character'
    )
    list_filter = (
        'corporation_name',
        'alliance_name',
        'faction_name',
        (
            'character_ownership__user__profile__main_character',
            admin.RelatedOnlyFieldListFilter
        ),
    )
    ordering = ('character_name', )

    def has_change_permission(self, request, obj=None):
        return False

    @staticmethod
    def user(obj):
        try:
            return obj.character_ownership.user
        except (AttributeError, ObjectDoesNotExist):
            return None

    @staticmethod
    def main_character(obj):
        try:
            return obj.character_ownership.user.profile.main_character
        except (AttributeError, ObjectDoesNotExist):
            return None

    def get_form(self, request, obj=None, **kwargs):
        if not obj or not obj.pk:
            return EveCharacterForm
        return super().get_form(request, obj=obj, **kwargs)



from django.urls import include, path

from . import views

app_name = 'discord'

module_urls = [
    # Discord Service Control
    path('activate/', views.activate_discord, name='activate'),
    path('deactivate/', views.deactivate_discord, name='deactivate'),
    path('reset/', views.reset_discord, name='reset'),
    path('callback/', views.discord_callback, name='callback'),
    path('add_bot/', views.discord_add_bot, name='add_bot'),
]

urlpatterns = [
    path('discord/', include((module_urls, app_name), namespace=app_name))
]



# Generated by Django 3.2.8 on 2021-10-17 16:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0004_auto_20211015_0502'),
    ]

    operations = [
        migrations.AlterField(
            model_name='analyticspath',
            name='ignore_path',
            field=models.CharField(default='/example/', help_text='Regex Expression, If matched no Analytics Page View is sent', max_length=254),
        ),
    ]
